@@ -14,6 +14,7 @@
 (defvar inhibit-eol-conversion nil)
 (defvar inhibit-point-motion-hooks nil)
 (defvar jka-compr-inhibit nil)
+(defvar large-file-warning-threshold nil)
 (defvar last-pressed-toolbar-button nil)
 (defvar longlines-mode nil)
 (defvar mark-active nil)
@@ -45,6 +46,7 @@
 (defun ange-ftp-file-name-all-completions (&rest args))
 (defun ange-ftp-file-name-completion (&rest args))
 (defun bitmap-insert-xbm-file (&rest args))
+(defun charset-priority-list (&rest args))
 (defun create-image (&rest args))
 (defun define-process-argument-editing (&rest args))
 (defun delete-overlay (&rest args))
@@ -83,9 +85,11 @@
 (defun posn-window (&rest args))
 (defun profile-functions (&rest args))
 (defun propertize (&rest args))
+(defun propertized-buffer-identification (&rest args))
 (defun remove-images (&rest args))
 (defun set-buffer-menubar (&rest args))
 (defun set-buffer-multibyte (&rest args))
+(defun set-charset-priority (&rest args))
 (defun set-coding-priority (&rest args))
 (defun set-process-query-on-exit-flag (&rest args))
 (defun set-toolbar-button-down-flag (&rest args))
