@@ -37,35 +37,35 @@
 (defvar mew-vinfo nil) ;; Virtual mode
 (defvar mew-xinfo nil) ;; Analyzed
 
-(mapcar 'make-variable-buffer-local
-	(list 'mew-summary-buffer-process
-	      'mew-summary-buffer-process-status
-	      'mew-summary-buffer-process-error
-	      'mew-summary-buffer-secure-process
-	      'mew-summary-buffer-left-msgs
-	      'mew-summary-buffer-raw
-	      'mew-decode-syntax
-	      'mew-encode-syntax
-	      'mew-ainfo
-	      'mew-cinfo
-	      'mew-dinfo
-	      'mew-minfo
-	      'mew-sinfo
-	      'mew-tinfo
-	      'mew-vinfo
-	      'mew-xinfo))
+(mapc 'make-variable-buffer-local
+      (list 'mew-summary-buffer-process
+	    'mew-summary-buffer-process-status
+	    'mew-summary-buffer-process-error
+	    'mew-summary-buffer-secure-process
+	    'mew-summary-buffer-left-msgs
+	    'mew-summary-buffer-raw
+	    'mew-decode-syntax
+	    'mew-encode-syntax
+	    'mew-ainfo
+	    'mew-cinfo
+	    'mew-dinfo
+	    'mew-minfo
+	    'mew-sinfo
+	    'mew-tinfo
+	    'mew-vinfo
+	    'mew-xinfo))
 
 (provide 'mew-blvs)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2006 Mew developing team.
+;; Copyright (C) 2000-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -74,7 +74,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
