@@ -11,7 +11,7 @@
   '(("B" . mew-base64-encode-string)
     ("Q" . mew-q-encode-string)))
 
-(defsubst mew-header-encode-get-func (b-or-q)
+(defun mew-header-encode-get-func (b-or-q)
   (cdr (mew-assoc-case-equal b-or-q mew-header-encode-switch 0)))
 
 
@@ -25,13 +25,13 @@
      nil
      ,mew-error-invalid-q-encoding)))
 
-(defsubst mew-header-decode-get-func (b-or-q)
+(defun mew-header-decode-get-func (b-or-q)
   (nth 1 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
-(defsubst mew-header-decode-get-error-func (b-or-q)
+(defun mew-header-decode-get-error-func (b-or-q)
   (nth 2 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
-(defsubst mew-header-decode-get-error-str (b-or-q)
+(defun mew-header-decode-get-error-str (b-or-q)
   (nth 3 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
 (defconst mew-header-decode-regex
@@ -141,11 +141,11 @@
 ;;; Quoted-printable encoding
 ;;;
 
-(defun mew-q-encode-string (str)
+(defun mew-q-encode-string (str &optional prefix)
   (let* ((len (length str))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char)
-    (while (< i len)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char)
+    (dotimes (i len)
       (setq char (aref str i))
       (cond
        ((char-equal char mew-sp)
@@ -154,28 +154,29 @@
 	     (< char 126)
 	     (not (char-equal char ?=))
 	     (not (char-equal char ??))
-	     (not (char-equal char ?_))) ;; space
+	     (not (char-equal char ?_)) ;; space
+	     (or (not prefix) (not (memq char '(?# ?@)))))
 	(aset ret j char))
        (t
-	(aset ret j ?=)
+	(aset ret j (or prefix ?=))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (logand char 15)))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (substring ret 0 j)))
 
 (defun mew-samba-encoding (str)
   (setq str (mew-cs-encode-string str mew-cs-samba))
   (let* ((len (length str))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char type)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char type)
     (cond
      ((eq mew-use-samba-encoding-type 'cap)
       (setq type 'cap))
      (t
       (setq type 'hex)))
-    (while (< i len)
+    (dotimes (i len)
       (setq char (aref str i))
       (if (and (eq type 'cap) (< char 128))
 	  (aset ret j char)
@@ -184,7 +185,7 @@
 	(aset ret j (aref "0123456789abcdef" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789abcdef" (logand char 15))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (substring ret 0 j)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -192,23 +193,23 @@
 ;;; Quoted-printable decoding
 ;;;
 
-(defsubst mew-hexchar-to-int (hex)
+(defun mew-hexchar-to-int (hex)
   ;; would return nil if error
   (cond
    ((and (<= ?0 hex) (<= hex ?9)) (- hex ?0))
    ((and (<= ?A hex) (<= hex ?F)) (+ (- hex ?A) 10))
    ((and (<= ?a hex) (<= hex ?f)) (+ (- hex ?a) 10))))
 
-(defun mew-q-decode-string (qpstr &optional key)
+(defun mew-q-decode-string (qpstr &optional okey)
   (condition-case nil
       (let* ((len (length qpstr))
-	     (ret (make-string len ?a))
-	     (i 0) (j 0) char)
-	(setq key (or key ?=))
-	(while (< i len)
+	     (ret (mew-make-string len))
+	     (j 0) char key)
+	(setq key (or okey ?=))
+	(dotimes (i len)
 	  (setq char (aref qpstr i))
 	  (cond
-	   ((char-equal char ?_)
+	   ((and (not okey) (char-equal char ?_))
 	    (aset ret j mew-sp))
 	   ((char-equal char key)
 	    (aset ret j (+ (* (mew-hexchar-to-int (aref qpstr (1+ i))) 16)
@@ -216,7 +217,7 @@
 	    (setq i (+ i 2)))
 	   (t
 	    (aset ret j char)))
-	  (setq i (1+ i) j (1+ j)))
+	  (setq j (1+ j)))
 	(substring ret 0 j))
     (error nil)))
 
@@ -254,7 +255,7 @@
 	(mew-encode-error
 	 (format "Unknown coding system %s in the header" (symbol-name hcs)))
       (setq charset (mew-cs-to-charset hcs))
-      (setq b-or-q (mew-ecsdb-b-or-q ecsdb))
+      (setq b-or-q (mew-ecsdb-get-b-or-q ecsdb))
       (setq fun (mew-header-encode-get-func b-or-q))
       (if (null fun)
 	  (mew-encode-error
@@ -294,10 +295,9 @@
   (let ((str-list (mapcar 'mew-chop (mew-split str ?,))))
     (mew-header-encode-text (car str-list))
     (setq str-list (cdr str-list))
-    (while str-list
+    (dolist (sl str-list)
       (insert ", ") ;; must be fold here
-      (mew-header-encode-text (car str-list))
-      (setq str-list (cdr str-list)))))
+      (mew-header-encode-text sl))))
 
 (defmacro mew-header-encode-cond (c)
   `(cond
@@ -612,9 +612,9 @@
     ;; If each line does not end with ",", unfold it.
     ;; In Page 5 of RFC822 says, "Unfolding is accomplished by
     ;; regarding CRLF immediately followed by a LWSP-char as
-    ;; equivalent to the LWSP-char". However, it also says, 
-    ;; "In structured field bodies, multiple linear space ASCII 
-    ;; characters (namely HTABs and SPACEs) are treated as single 
+    ;; equivalent to the LWSP-char". However, it also says,
+    ;; "In structured field bodies, multiple linear space ASCII
+    ;; characters (namely HTABs and SPACEs) are treated as single
     ;; spaces and may freely surround any symbol." So, remove
     ;; continuous white spaces.
     (when (and mew-decode-broken (looking-at "\n[ \t]+"))
@@ -625,6 +625,14 @@
     (goto-char (point-min))
     (while (re-search-forward "\\([^,]\\)[ \t]*\n[ \t]+" nil t)
       (replace-match "\\1 ")))
+   ((eq type 'struct2) ;; for Authentication-Results:
+    (while (re-search-forward "[ \t]*\n[ \t]+" nil t) (replace-match " "))
+    (goto-char (point-min))
+    (while (search-forward ";" nil t)
+      (insert "\n")
+      (if (looking-at "[\t ]+")
+	  (replace-match "\t")
+	(insert "\t"))))
    ((eq type 'text)
     ;; In Page 5 of RFC822 says, "Unfolding is accomplished by
     ;; regarding CRLF immediately followed by a LWSP-char as
@@ -634,8 +642,8 @@
    (t ;; unstruct
     )))
 
-(defsubst mew-header-decode-string (key)
-  (let ((beg (match-beginning 0)) 
+(defun mew-header-decode-string (key)
+  (let ((beg (match-beginning 0))
 	(end (match-end 0))
 	(css (mew-header-decode (mew-match-string 1)
 				(mew-match-string 2)
@@ -645,17 +653,17 @@
     (delete-region beg end)
     (insert css)))
 
-(defsubst mew-header-decode-in-quotedp ()
+(defun mew-header-decode-in-quotedp ()
   (save-excursion
     (beginning-of-line)
     (char-equal (following-char) ?\")))
 
-(defsubst mew-header-decode-in-addr-spec ()
+(defun mew-header-decode-in-addr-spec ()
   (eq (get-text-property (match-beginning 0) 'mew-addr-spec) t))
 
 (defun mew-decode-warning-body (err msg)
   (mew-xinfo-set-warning
-   (cons 
+   (cons
     (cond
      ((eq err 'unknown-charset)
       (format "Charset (%s) for body is not supported.\n" msg))
@@ -699,7 +707,9 @@
        ((eq err 'ctl)
 	(concat "The '" param "' has invalid control codes.\n"))
        ((eq err 'enc)
-	(concat "The '" param "' parameter has encoded-word.\n")))
+	(concat "The '" param "' parameter has encoded-word.\n"))
+       ((eq err 'char)
+	(concat "The '" param "' has invalid characters.\n")))
       (mew-xinfo-get-warning)))))
 
 (defun mew-header-decode-region (key rbeg rend &optional draftp)
@@ -722,9 +732,9 @@ That is, each line may be more than 75."
       ;;
       (goto-char (point-min))
       (when (re-search-forward mew-header-decode-regex nil t)
-	;; In Page 10 of RFC 2047 says, "When displaying a particular 
-	;; header field that contains multiple 'encoded-word's, any 
-	;; 'linear-white-space' that separates a pair of adjacent 
+	;; In Page 10 of RFC 2047 says, "When displaying a particular
+	;; header field that contains multiple 'encoded-word's, any
+	;; 'linear-white-space' that separates a pair of adjacent
 	;; 'encoded-word's is ignored". So, use looking-at.
 	(cond
 	 ((memq type '(text comma-text))
@@ -743,7 +753,7 @@ That is, each line may be more than 75."
 	  ;; encoded-word in quoted-string should not be decoded
 	  ;; according to RFC 2047. However, if users wish
 	  ;; (ie mew-decode-broken is *non-nil*), decode it.
-	  ;; Regular expression cannot express a quoted string in 
+	  ;; Regular expression cannot express a quoted string in
 	  ;; general case. So, canonicalize as follows:
 	  ;;     Not quoted line (may be null line)
           ;;     "quoted line"
@@ -825,9 +835,9 @@ That is, each line may be more than 75."
 	 (charset (mew-cs-to-charset hcs))
          (estr (mew-cs-encode-string str hcs))
 	 (len (length estr))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char)
-    (while (< i len)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char)
+    (dotimes (i len)
       (setq char (aref estr i))
       (if (or (and (<= ?0 char) (<= char ?9))
 	      (and (<= ?a char) (<= char ?z))
@@ -838,7 +848,7 @@ That is, each line may be more than 75."
 	(aset ret j (aref "0123456789ABCDEF" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (logand char 15))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (concat charset "''" (substring ret 0 j))))
 
 (defun mew-param-decode (whole-value)
@@ -846,23 +856,21 @@ That is, each line may be more than 75."
 	 (value (car value-params))
 	 (params (cdr value-params))
 	 (max 0) num
-	 ret ext ea ext-sort entry charset cs
+	 ret ext ext-sort entry charset cs
 	 paramname-value paramname paramvalue)
-    (while params
-      (setq paramname-value (mew-param-analyze (car params)))
+    (dolist (param params)
+      (setq paramname-value (mew-param-analyze param))
       (when paramname-value
 	(setcar paramname-value (downcase (nth 0 paramname-value)))
 	(if (= (length paramname-value) 2)
 	    (setq ret (cons paramname-value ret))
 	  (setq ext (cons paramname-value ext))
 	  (setq num (nth 2 paramname-value))
-	  (if (> num max) (setq max num))))
-      (setq params (cdr params)))
+	  (if (> num max) (setq max num)))))
     (if (null ext)
 	(cons value (nreverse ret)) ;; fast return
       (setq max (1+ max))
-      (while ext
-	(setq ea (car ext))
+      (dolist (ea ext)
 	(mew-set '(paramname paramvalue num) ea)
 	(if (setq entry (assoc paramname ext-sort))
 	    (progn
@@ -874,10 +882,8 @@ That is, each line may be more than 75."
 	  (aset entry num paramvalue)
 	  (setq ext-sort
 		(cons (list paramname entry (nth 3 ea) (nth 4 ea))
-		      ext-sort)))
-	(setq ext (cdr ext)))
-      (while ext-sort
-	(setq ea (car ext-sort))
+		      ext-sort))))
+      (dolist (ea ext-sort)
 	(setq paramvalue nil)
 	(mew-set '(paramname entry charset) ea)
 	(setq num 0)
@@ -902,15 +908,14 @@ That is, each line may be more than 75."
 	  (if (mew-coding-system-p cs)
 	      (setq paramvalue (mew-cs-decode-string paramvalue cs))
 	    (setq paramvalue mew-error-unknown-charset)))
-	(setq ret (cons (list paramname paramvalue) ret))
-	(setq ext-sort (cdr ext-sort)))
+	(setq ret (cons (list paramname paramvalue) ret)))
       (setq ret (mapcar 'mew-param-sanity-check ret))
       (cons value (nreverse ret))))) ;; late return
 
 (defun mew-param-analyze (param)
-  "Return (paramname paramvalue) or 
+  "Return (paramname paramvalue) or
           (paramname paramvalue section charset lang)."
-  (let* (name section asterisk value charset lang)
+  (let (name section asterisk value charset lang)
     (when (string-match mew-header-param-regex param)
       (setq name (match-string 1 param))
       (setq section (match-string 2 param)) ;; *21
@@ -931,12 +936,16 @@ That is, each line may be more than 75."
 	    (if mew-decode-broken
 		(setq value (mew-param-analyze-broken value name))))
 	(when (and (= section 0)
-		   (string-match "^\\([^']*\\)'\\([^']*\\)'\\(.*\\)$" value))
+		   ;; broken UAs quote extended-value, sigh
+		   (string-match "^\"?\\([^']*\\)'\\([^']*\\)'\\([^\"]*\\)\"?$" value))
 	  (setq charset (match-string 1 value))
 	  (setq lang (match-string 2 value))
 	  (if (string= lang "") (setq lang nil))
 	  (setq value (match-string 3 value))
 	  (if (string= value "") (setq value nil)))
+	;; broken UAs quote extended-value, sigh
+	(if (string-match "^\"\\([^\"]*\\)\"$" value)
+	    (setq value (mew-match-string 1 value)))
 	(setq value (mew-q-decode-string value ?%)))
       (if section
 	  (list name value section charset lang)
@@ -971,17 +980,200 @@ That is, each line may be more than 75."
     (unless (string= value new) (mew-decode-warning-params param 'ctl))
     (list param new)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Punycode:: RFC 3492
+;;;
+
+(defconst mew-puny-base 36)
+(defconst mew-puny-tmin 1)
+(defconst mew-puny-tmax 26)
+(defconst mew-puny-damp 700)
+(defconst mew-puny-skew 38)
+(defconst mew-puny-initial-bias 72)
+(defconst mew-puny-initial-n 128)
+(defconst mew-puny-delimiter ?-)
+
+(defun mew-puny-adapt (delta numpoints firsttime)
+  (let ((k 0))
+    (if firsttime
+	(setq delta (/ delta mew-puny-damp))
+      (setq delta (/ delta 2)))
+    (setq delta (+ delta (/ delta numpoints)))
+    (while (> delta (/ (* (- mew-puny-base mew-puny-tmin) mew-puny-tmax) 2))
+      (setq delta (/ delta (- mew-puny-base mew-puny-tmin)))
+      (setq k (+ k mew-puny-base)))
+    (+ k (/ (* (1+ (- mew-puny-base mew-puny-tmin)) delta) (+ delta mew-puny-skew)))))
+
+(defun mew-puny-decode-digit (cp)
+  (if (< (- cp 48) 10)
+      (- cp 22)
+    (if (< (- cp 65) 26)
+	(- cp 65)
+      (if (< (- cp 97) 26)
+	  (- cp 97)
+	mew-puny-base))))
+
+(defun mew-puny-encode-digit (d)
+  (if (< d 26)
+      (+ d 22 75) ;; a-z
+    (+ d 22))) ;; 0-9
+
+(defun mew-puny-decode (input)
+  (when mew-cs-utf-16be
+    (condition-case nil
+	(mew-cs-decode-string
+	 (mew-puny-decode1 (substring input 4)) ;; xn--
+	 mew-cs-utf-16be)
+      (error nil))))
+
+(defun mew-puny-decode1 (input)
+  (let* ((n mew-puny-initial-n)
+	 (bias mew-puny-initial-bias)
+	 (len (length input))
+	 (in 0) (out 0)
+	 (i 0) (b 0)
+	 digit thr oldi w k output ret)
+    (dotimes (j len)
+      (if (= (aref input j) mew-puny-delimiter) (setq b j)))
+    (dotimes (j b)
+      (setq output (cons (aref input j) output))
+      (setq out (1+ out)))
+    (setq output (nreverse output))
+    (if (> b 0) (setq in (1+ b)) (setq in 0))
+    (while (< in len)
+      (setq oldi i)
+      (setq w 1)
+      (setq k mew-puny-base)
+      (catch 'loop
+	(while t
+	  (if (>= in len) (error "punycode bad input"))
+	  (setq digit (mew-puny-decode-digit (aref input in)))
+	  (if (>= digit mew-puny-base) (error "punycode bad input"))
+	  (setq in (1+ in))
+	  (setq i (+ i (* digit w)))
+	  (if (<= k bias)
+	      (setq thr mew-puny-tmin)
+	    (if (>= k (+ bias mew-puny-tmax))
+		(setq thr mew-puny-tmax)
+	      (setq thr (- k bias))))
+	  (if (< digit thr) (throw 'loop nil))
+	  (setq w (* w (- mew-puny-base thr)))
+	  (setq k (+ k mew-puny-base))))
+      (setq out (1+ out))
+      (setq bias (mew-puny-adapt (- i oldi) out (= oldi 0)))
+      (setq n (+ n (/ i out)))
+      (setq i (% i out))
+      (if (= i 0)
+	  (setq output (cons n (nthcdr i output)))
+	(setcdr (nthcdr (1- i) output) (cons n (nthcdr i output))))
+      (setq i (1+ i)))
+    (setq ret (mew-make-string (* out 2)))
+    (let ((j 0))
+      (dolist (op output)
+	(aset ret j (/ op 256))
+	(setq j (1+ j))
+	(aset ret j (% op 256))
+	(setq j (1+ j))))
+    ret))
+
+(defun mew-puny-encode (input)
+  (when mew-cs-utf-16be
+    (condition-case nil
+ 	(concat "xn--" (mew-puny-encode1 (mew-cs-encode-string input mew-cs-utf-16be)))
+      (error nil))))
+
+(defun mew-puny-encode1 (input)
+  (let* ((len (length input))
+	 (h-len (/ len 2))
+	 (n mew-puny-initial-n)
+	 (bias mew-puny-initial-bias)
+	 (delta 0) (out 0)
+	 (output (mew-make-string (* len 4)))
+	 h b m q k thr uni)
+    (dotimes (j len)
+      (setq uni (aref input j))
+      (setq j (1+ j))
+      (setq uni (+ (* uni 256) (aref input j)))
+      (when (< uni 128) ;; basic
+	(aset output out uni)
+	(setq out (1+ out))))
+    (setq h out)
+    (setq b out)
+    (when (> b 0)
+      (aset output out mew-puny-delimiter)
+      (setq out (1+ out)))
+    (while (< h h-len)
+      (setq m 65536) ;; 17bits
+      (dotimes (j len)
+	(setq uni (aref input j))
+	(setq j (1+ j))
+	(setq uni (+ (* uni 256) (aref input j)))
+	(if (and (>= uni n) (< uni m)) (setq m uni)))
+      (setq delta (+ delta (* (- m n) (1+ h))))
+      (setq n m)
+      (dotimes (j len)
+	(setq uni (aref input j))
+	(setq j (1+ j))
+	(setq uni (+ (* uni 256) (aref input j)))
+	(when (< uni n)
+	  (setq delta (1+ delta))
+	  (if (= delta 0) (error "punycode overflow")))
+	(when (= uni n)
+	  (setq q delta)
+	  (setq k mew-puny-base)
+	  (catch 'loop
+	    (while t
+	      (if (<= k bias)
+		  (setq thr mew-puny-tmin)
+		(if (>= k (+ bias mew-puny-tmax))
+		    (setq thr mew-puny-tmax)
+		  (setq thr (- k bias))))
+	      (if (< q thr) (throw 'loop nil))
+	      (aset output out (mew-puny-encode-digit (+ thr (% (- q thr) (- mew-puny-base thr)))))
+	      (setq out (1+ out))
+	      (setq q (/ (- q thr) (- mew-puny-base thr)))
+	      (setq k (+ k mew-puny-base))))
+	  (aset output out (mew-puny-encode-digit q))
+	  (setq out (1+ out))
+	  (setq bias (mew-puny-adapt delta (1+ h) (= h b)))
+	  (setq delta 0)
+	  (setq h (1+ h))))
+      (setq delta (1+ delta))
+      (setq n (1+ n)))
+    (substring output 0 out)))
+
+(defun mew-puny-encode-url (url)
+  (let (beg end idn)
+    (with-temp-buffer
+      (insert url)
+      (goto-char (point-min))
+      (if (search-forward "://" nil t)
+	  (setq beg (point))
+	(setq beg (point-min)))
+      (if (search-forward "/" nil t)
+	  (setq end (1- (point)))
+	(setq end (point-max)))
+      (save-restriction
+	(narrow-to-region beg end)
+	(goto-char (point-min))
+	(while (re-search-forward "[^.]?[^.\000-\177][^.]*" nil t)
+	  (setq idn (mew-match-string 0))
+	  (delete-region (match-beginning 0) (match-end 0))
+	  (insert (mew-puny-encode idn))))
+      (mew-buffer-substring (point-min) (point-max)))))
+
 (provide 'mew-bq)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2006 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -990,7 +1182,7 @@ That is, each line may be more than 75."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
