@@ -41,10 +41,10 @@
 
 (mew-blinfo-defun 'mew-xinfo mew-xinfo-list)
 
-(defsubst mew-xinfo-copy (buf)
+(defun mew-xinfo-copy (buf)
   (set 'mew-xinfo (save-excursion (set-buffer buf) (symbol-value 'mew-xinfo))))
 
-(defsubst mew-xinfo-clear ()
+(defun mew-xinfo-clear ()
   (set 'mew-xinfo nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -52,23 +52,23 @@
 ;;  Prepare new message --- caching
 ;;
 
-(defsubst mew-cache-decode-syntax (buf)
+(defun mew-cache-decode-syntax (buf)
   (save-excursion (set-buffer buf) mew-decode-syntax))
 
 (defvar mew-cache nil
-  "A list of decoded messages cache. 
+  "A list of decoded messages cache.
 The (new ... old) order of ((\"+folder\" . \"message\") . cache-buffer)")
 
-(defsubst mew-cache-key (fld msg)
+(defun mew-cache-key (fld msg)
   (cons fld msg))
 
-(defsubst mew-cache-buffer-get (entry)
+(defun mew-cache-buffer-get (entry)
   (cdr entry))
 
-(defsubst mew-cache-entry-make (fld msg buf)
+(defun mew-cache-entry-make (fld msg buf)
   (cons (mew-cache-key fld msg) buf))
 
-(defsubst mew-cache-get (fld msg)
+(defun mew-cache-get (fld msg)
   (assoc (mew-cache-key fld msg) mew-cache))
 
 (defun mew-cache-hit (fld msg &optional must-hit)
@@ -83,7 +83,7 @@ If MUST-HIT is non-nil and no valid cache entry is found, an error occurs."
 	    (error "Cache not found")
 	  nil)
       (setq cache (mew-cache-buffer-get entry))
-      (setq file (mew-expand-folder fld msg))
+      (setq file (mew-expand-msg fld msg))
       (unless (file-readable-p file)
 	(error "%s does not exist" (mew-concat-folder fld msg)))
       (setq time (mew-file-get-time file))
@@ -137,21 +137,29 @@ If MUST-HIT is non-nil and no valid cache entry is found, an error occurs."
 
 (defun mew-xinfo-get-xcc (fld msg)
   (when (or (mew-folder-queuep fld) (mew-folder-postqp fld))
-    (let ((info (concat (mew-expand-folder fld msg) mew-queue-info-suffix))
+    (let ((info (concat (mew-expand-msg fld msg) mew-queue-info-suffix))
 	  (headers mew-fields-xcc)
-	  data header addr xinfo)
+	  data addr xinfo)
       (when (and (file-readable-p info)
 		 (setq data (aref (mew-lisp-load info) 0)))
 	(with-temp-buffer
 	  (insert data)
 	  (goto-char (point-min))
-	  (while headers
-	    (setq header (car headers))
-	    (setq headers (cdr headers))
+	  (dolist (header headers)
 	    (when (setq addr (mew-header-get-value header))
 	      (setq xinfo (cons (format "%s %s\n" header addr) xinfo)))))
 	(nreverse xinfo)))))
 
+(defun mew-disable-alternative-check ()
+  (let ((xmailer (mew-header-get-value mew-x-mailer:))
+	(regexs mew-disable-alternative-regex-list))
+    (if (null xmailer)
+	t
+      (catch 'loop
+	(dolist (regex regexs t)
+	  (if (string-match regex xmailer)
+	      (throw 'loop nil)))))))
+
 (defun mew-cache-message (fld msg &optional unlimit no-err)
   "Cache the message specified by FLD and MSG.
 If an invalid message are cached, deletes it and caches the message again.
@@ -160,6 +168,7 @@ the limitations. If NO-ERR is non-nil, an error is caused
 if decode fails."
   (let* ((cbuf (current-buffer))
 	 (cache (mew-cache-hit fld msg))
+	 (use-alternative mew-use-alternative)
 	 tim-siz decode errormsg)
     (catch 'return
       (if cache
@@ -186,7 +195,9 @@ if decode fails."
 	 (setq errormsg (nth 1 errmsg))
 	 (throw 'return (setq cache nil))))
       (mew-cinfo-set fld msg (car tim-siz) (cdr tim-siz) mew-decode-broken)
-      (mew-dinfo-set nil t t mew-use-alternative)
+      (if (and use-alternative mew-disable-alternative-regex-list)
+	  (setq use-alternative (mew-disable-alternative-check)))
+      (mew-dinfo-set nil t t use-alternative)
       (mew-decode-syntax-clear)
       (mew-xinfo-set-text-body mew-use-text-body)
       (condition-case nil
@@ -200,7 +211,8 @@ if decode fails."
 	(quit
 	 ;; prefetching an encrypted message
 	 (mew-cache-delete)
-	 (message "MIME decoding for %s/%s aborted" fld msg)
+	 ;; The following message is not friendly to users.
+	 ;; (message "MIME decoding for %s/%s aborted" fld msg)
 	 (throw 'return (setq cache nil))))
       (mew-ainfo-set-icon msg)
       (mew-xinfo-set-info (append (mew-xinfo-get-info) (mew-xinfo-get-xcc fld msg)))
@@ -213,10 +225,8 @@ if decode fails."
 (defun mew-cache-clean-up ()
   "A function to flush all decoded messages in cache list."
   (interactive)
-  (let ((n 0))
-    (while (< n mew-cache-size)
-      (mew-kill-buffer (format "%s%d" mew-buffer-cache n))
-      (setq n (1+ n))))
+  (dotimes (n mew-cache-size)
+    (mew-kill-buffer (format "%s%d" mew-buffer-cache n)))
   (mew-summary-reset)
   (setq mew-cache nil))
 
@@ -226,13 +236,13 @@ if decode fails."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -241,7 +251,7 @@ if decode fails."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
