@@ -22,7 +22,7 @@
 ;;; Readability
 ;;;
 
-(defconst mew-sp ?\ ) ;; 32 does not work on XEmacs
+(defconst mew-sp ?\ )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -113,8 +113,12 @@
 (defconst mew-ct-apk "Application/Pgp-Keys")
 (defconst mew-ct-pgs "application/pgp-signature") ;; due to the RFC 1847 bug
 (defconst mew-ct-pge "application/pgp-encrypted") ;; due to the RFC 1847 bug
-(defconst mew-ct-sms "application/pkcs7-signature")
-(defconst mew-ct-xsms "application/x-pkcs7-signature")
+(defconst mew-ct-smm "Application/Pkcs7-Mime")
+(defconst mew-ct-xsmm "Application/X-Pkcs7-Mime")
+(defconst mew-ct-smm-enc "enveloped-data")
+(defconst mew-ct-smm-sig "signed-data")
+(defconst mew-ct-sms "application/pkcs7-signature")  ;; due to the RFC 1847 bug
+(defconst mew-ct-xsms "application/x-pkcs7-signature")  ;; due to the RFC 1847 bug
 
 (defconst mew-us-ascii "us-ascii")
 (defconst mew-utf-8 "utf-8")
@@ -137,7 +141,7 @@
 (defconst mew-bin  "binary")
 (defconst mew-xuu  "x-uuencode")
 
-(defconst mew-decode-value 
+(defconst mew-decode-value
   `(,mew-b64 ,mew-qp ,mew-xg ,mew-7bit ,mew-8bit ,mew-bin ,mew-xuu))
 (defconst mew-decode-composite-value
   `(,mew-7bit ,mew-8bit ,mew-bin))
@@ -160,16 +164,11 @@
   (+ (length [key beg end pri]) (length mew-mime-fields)))
 
 (defconst mew-x-mailer
-  (concat mew-version " on "
-          (if (string-match "XEmacs" emacs-version) "XEmacs" "Emacs")
-          (if (boundp 'emacs-program-version)
-              (format " %s " emacs-program-version)
-	    (string-match "^\\([.0-9]+\\)\\.[0-9]+$" emacs-version)
-	    (concat " " (match-string 1 emacs-version) " "))
-          (and (boundp 'xemacs-codename)
-	       (string-match "^[a-zA-Z0-9 ]+$" xemacs-codename)
-	       (concat "(" xemacs-codename ")"))
-          (and (boundp 'mule-version) (concat "/ Mule " mule-version)))
+  (format "%s on Emacs %s / Mule %s"
+	  mew-version
+	  (when (string-match "^\\([.0-9]+\\)\\.[0-9]+$" emacs-version)
+	    (match-string 1 emacs-version))
+	  mule-version)
   "*A value inserted into X-Mailer: field in Draft mode if *non-nil*.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -219,13 +218,13 @@
 (defconst mew-regex-mark "^\\([^ \n]\\)")
 (defun mew-sumsyn-mark () (string-to-char (mew-match-string 1)))
 
-;; fld msg 
+;; fld msg
 (defconst mew-regex-sumsyn-short   "\r \\([^ \n]*\\) \\([0-9]+\\)")
 
 ;; fld msg my-id par-id uid siz
 (defconst mew-regex-sumsyn-long    "\r \\([^ \n]*\\) \\([0-9]+\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([0-9]*\\)")
 
-;; fld msg 
+;; fld msg
 (defun mew-regex-sumsyn-msg (msg)
                            (format "\r \\([^ \n]*\\) \\(%s\\) " msg))
 
@@ -285,6 +284,8 @@
 (defconst mew-regex-case1 "^\\([^-+$%*,:][^,:]+\\(,[^,:]+\\)*\\):\\(.*\\)")
 (defconst mew-regex-case2 "\\([^-+$%*,:][^,:]+\\(,[^,:]+\\)*\\):")
 
+(defconst mew-regex-punycode "xn--\\([-0-9a-zA-z]+\\)")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder
@@ -322,7 +323,8 @@
 (defconst mew-inherit-7bit nil)
 (defconst mew-inherit-offline nil)
 (defconst mew-inherit-decode-signer nil)
-(defconst mew-inherit-encode-signer nil)
+(defconst mew-inherit-encode-pgp-signer nil)
+(defconst mew-inherit-encode-smime-signer nil)
 (defconst mew-inherit-prefetching nil)
 (defconst mew-inherit-refile-case nil)
 (defconst mew-inherit-refile-proto nil)
@@ -332,19 +334,21 @@
 (defconst mew-inherit-pick-ret nil)
 (defconst mew-inherit-pick-mewlp nil)
 (defconst mew-inherit-pick-omit-and nil)
+(defconst mew-inherit-pick-omit-and2 nil)
 (defconst mew-inherit-submission nil)
+(defconst mew-inherit-complete-folder nil)
 
 (provide 'mew-const)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2006 Mew developing team.
+;; Copyright (C) 2000-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -353,7 +357,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
