@@ -12,19 +12,33 @@
 ;;; Decode info
 ;;;
 
-;; decode-body:
-;; t means decode with MIME encoding and convert charset.
-;; symbol means decode with MIME encoding only.
+;; limit:
+;;   Depth of RFC 822 messages.
+;;   The cnt starts with 0.
+;;   Decode if cnt <= limit
+;;   nil means that limit is 0.
+;;
+;; decode-text:
+;;   If t, decode a text MIME body.
+;;   Otherwise, it is treated as 'no-cs-conv
+;;
+;; decode-binary:
+;;   If t, decode a binary (non-text) MIME body.
+;;
+;; use-alt:
+;;   If t, one part in Multipart/Alternative is used.
+;;   Otherwise, it is treated as Multipart/Mixed.
 
-(defvar mew-dinfo-list '("limit" "decode-text" "decode-binary" "use-alt"))
+(defvar mew-dinfo-list '("limit" "decode-text" "decode-binary" "use-alt" "encap-html"))
 
 (mew-blinfo-defun 'mew-dinfo mew-dinfo-list)
 
-(defun mew-dinfo-set (lim dt db alt)
+(defun mew-dinfo-set (lim dt db alt &optional encap)
   (mew-dinfo-set-limit lim)
   (mew-dinfo-set-decode-text dt)
   (mew-dinfo-set-decode-binary db)
-  (mew-dinfo-set-use-alt alt))
+  (mew-dinfo-set-use-alt alt)
+  (mew-dinfo-set-encap-html encap))
 
 (defun mew-cache-dinfo-get-use-alt (buf)
   (when buf
@@ -49,26 +63,26 @@
     (,mew-xg  "-d" "-g" "-t")
     (,mew-xuu "-d" "-u")))
 
-(defsubst mew-prog-mime-decode-get-opt (cte switch)
+(defun mew-prog-mime-decode-get-opt (cte switch)
   (cdr (mew-assoc-case-equal cte switch 0)))
 
 (defvar mew-decode-multipart-encrypted-switch
   `((,mew-ct-pge mew-pgp-decrypt mew-pgp-ver mew-prog-pgp)))
 
 (defvar mew-decode-multipart-signed-switch
-  `((,mew-ct-pgs mew-pgp-verify   mew-pgp-ver   mew-prog-pgp)
-    (,mew-ct-sms mew-smime-verify mew-smime-ver mew-prog-smime)
-    (,mew-ct-xsms mew-smime-verify mew-smime-ver mew-prog-smime)))
+  `((,mew-ct-pgs  mew-pgp-verify          mew-pgp-ver   mew-prog-pgp)
+    (,mew-ct-sms  mew-smime-detach-verify mew-smime-ver mew-prog-smime)
+    (,mew-ct-xsms mew-smime-detach-verify mew-smime-ver mew-prog-smime)))
 
 ;;
 
-(defsubst mew-decode-get-security-func (proto switch)
+(defun mew-decode-get-security-func (proto switch)
   (nth 1 (mew-assoc-case-equal proto switch 0)))
 
-(defsubst mew-decode-get-security-existence (proto switch)
+(defun mew-decode-get-security-existence (proto switch)
   (symbol-value (nth 2 (mew-assoc-case-equal proto switch 0))))
 
-(defsubst mew-decode-get-security-prog (proto switch)
+(defun mew-decode-get-security-prog (proto switch)
   (symbol-value (nth 3 (mew-assoc-case-equal proto switch 0))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -76,9 +90,9 @@
 ;;; MIME decoder
 ;;;
 
-(defsubst mew-decode-error (error-msg)
+(defun mew-decode-error (error-msg)
   (mew-xinfo-set-decode-err error-msg)
-  (error ""))
+  (error error-msg))
 
 ;;;
 ;;; Decoding a header
@@ -106,11 +120,11 @@
 over 'mew-header-veil-count' are covered with invisible veils.
 This commands toggles visibility of these lines."
   (interactive)
-  (when mew-use-header-veil
+  (when (and mew-use-header-veil (get-buffer (mew-buffer-message)))
     (mew-summary-msg-or-part
      (save-excursion
        (set-buffer (mew-buffer-message))
-       (when (mew-msghder-p)
+       (when (mew-msghdr-p)
 	 (let* ((win (get-buffer-window (current-buffer)))
 		(pos (window-start win)))
 	   (mew-toggle-header-veil (mew-minfo-get-veil-to))
@@ -155,7 +169,7 @@ This commands toggles visibility of these lines."
 	     (setq vs-end (next-single-property-change vs-beg 'mew-visible)))
 	    (mew-field-other-visible
 	     (setq vs-beg (next-single-property-change (point-min) 'mew-others))
-	     (if vs-beg 
+	     (if vs-beg
 		 (setq vs-end (next-single-property-change vs-beg 'mew-others))
 	       (setq vs-beg ch-beg)
 	       (setq vs-end ch-beg)))
@@ -179,8 +193,7 @@ This commands toggles visibility of these lines."
 	     ;; invisible
 	     (goto-char vs-beg)
 	     (setq cbeg (point))
-	     (insert contents)
-	     (put-text-property cbeg (point) 'mew-visible nil))) ;; for XEmacs
+	     (insert contents)))
 	   (goto-char (point-max)))
 	 (mew-decode-syntax-insert-privacy)
 	 (mew-decode-syntax-insert-warning)
@@ -213,10 +226,8 @@ This commands toggles visibility of these lines."
 	     (setq med (match-end 0))
 	     (forward-line)
 	     (setq nspec (mew-nspec-by-key key))
-	     (setq key-face (or (mew-nspec-keyface nspec)
-				'mew-face-header-key))
-	     (setq val-face (or (mew-nspec-valface nspec)
-				'mew-face-header-marginal))
+	     (setq key-face (mew-key-face key nspec))
+	     (setq val-face (mew-val-face key nspec))
 	     (put-text-property beg med 'face key-face)
 	     (put-text-property med (1- (point)) 'face val-face)
 	     (while (looking-at mew-lwsp+)
@@ -236,7 +247,9 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (visibles (make-list (length mew-field-spec) nil))
 	 (prop mew-use-highlight-header)
 	 key beg med subj from contents others
-	 key-face val-face N nspec visiblep mimep fld)
+	 key-face val-face N nspec visiblep
+	 mimep mimep2
+	 size ext ext-str ext-face)
     (mew-decode-narrow-to-header
      (while (not (eobp))
        (if (not (looking-at mew-keyval))
@@ -249,21 +262,22 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (setq nspec (mew-nspec-by-key key))
 	 (setq N (mew-nspec-n nspec))
 	 (setq visiblep (mew-nspec-visiblep nspec))
+	 (if (fboundp visiblep)
+	     (setq visiblep (funcall visiblep (mew-buffer-substring med (1- (point))))))
 	 (cond
 	  ((string= key mew-x-mew:)
 	   ;; deleting X-Mew: on the RFC822 header
 	   (delete-region beg (point)))
 	  ((string= key mew-x-mew-uidl:)
-	   (when (and (eq cnt 1)
-		      (setq fld (mew-cinfo-get-fld))
-		      (not (and mew-trash-folder (string= fld mew-trash-folder)))
-		      (not (mew-folder-remotep (mew-case:folder-folder fld))))
+	   (setq size (mew-scan-uid-size (mew-buffer-substring med (1- (point)))))
+	   (when (and (eq cnt 1) (mew-msg-truncatedp size))
 	     (mew-xinfo-set-action
-	      (mew-substitute-for-summary 
+	      (mew-substitute-for-summary
 	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\[mew-summary-retrieve-message]'"))))
 	  ((string-match "^Content-" key)
 	   ;; Due to PGP/MIME, properties are not put here.
 	   :; This must be "buffer-substring".
+	   (setq mimep2 t)
 	   (setq contents (cons (buffer-substring beg (point)) contents))
 	   (delete-region beg (point)))
 	  (t
@@ -286,10 +300,8 @@ Return the existence MIME-Version: and the value of Subject:."
 			  (mew-addrstr-parse-value
 			   (mew-buffer-substring med (point)))))))
 	   (when prop
-	     (setq key-face (or (mew-nspec-keyface nspec)
-				'mew-face-header-key))
-	     (setq val-face (or (mew-nspec-valface nspec)
-				'mew-face-header-marginal))
+	     (setq key-face (mew-key-face key nspec))
+	     (setq val-face (mew-val-face key nspec))
 	     (put-text-property beg med 'face key-face)
 	     (goto-char med)
 	     (forward-line)
@@ -297,7 +309,19 @@ Return the existence MIME-Version: and the value of Subject:."
 	     (while (looking-at mew-lwsp+)
 	       (forward-line)
 	       (put-text-property (match-end 0) (1- (point))
-				  'face val-face)))
+				  'face val-face))
+	     (when (setq ext (mew-nspec-extraface nspec))
+	       (save-restriction
+		 (narrow-to-region med (point))
+		 (while ext ;; cannot use dolist
+		   (goto-char (point-min))
+		   (setq ext-str (car ext))
+		   (setq ext (cdr ext))
+		   (setq ext-face (car ext))
+		   (setq ext (cdr ext))
+		   (while (re-search-forward ext-str nil t)
+		     (put-text-property (match-beginning 0) (match-end 0) 'face ext-face)))
+		 (goto-char (point-max)))))
 	   (cond
 	    ((null nspec) ;; others
 	     ;; This must be "buffer-substring".
@@ -315,25 +339,28 @@ Return the existence MIME-Version: and the value of Subject:."
     (put-text-property (point-min) (point) 'mew-noncontents t)
     (unless mew-field-other-visible
       (setq beg (point))
-      (mapcar 'insert (nreverse others))
+      (mapc 'insert (nreverse others))
       (put-text-property beg (point) 'mew-others t)
       (put-text-property beg (point) 'mew-noncontents t))
     (setq beg (point))
-    (mapcar (lambda (x) (and (stringp x) (insert x))) visibles)
+    (mapc (lambda (x) (and (stringp x) (insert x))) visibles)
     ;; for recenter in Message mode
     (put-text-property beg (point) 'mew-visible t)
     (put-text-property beg (point) 'mew-noncontents t)
     (mew-rear-nonsticky beg (point))
     (when mew-field-other-visible
       (setq beg (point))
-      (mapcar 'insert (nreverse others))
+      (mapc 'insert (nreverse others))
       (put-text-property beg (point) 'mew-others t)
       (put-text-property beg (point) 'mew-noncontents t)
       (mew-rear-nonsticky beg (point)))
     ;; the beginning of the content header
-    (save-excursion (mapcar 'insert (nreverse contents)))
+    (save-excursion (mapc 'insert (nreverse contents)))
     ;; 'mew-contents does not work due to PGP/MIME
-    (list mimep subj from)))
+    (if (and (null mimep) mimep2)
+	(mew-xinfo-set-warning
+	 (cons "No MIME-Version\n" (mew-xinfo-get-warning))))
+    (list (or mimep mimep2) subj from)))
 
 (defun mew-decode-mime-header (&optional dct)
   "A function to handle content header.
@@ -404,11 +431,11 @@ Return a part syntax after moving the beginning of the content body."
 	(setq mew-decode-syntax
 	      (mew-decode-message (mew-decode-syntax-rfc822-head) 0)))
     (condition-case nil
-	(setq mew-decode-syntax 
+	(setq mew-decode-syntax
 	      (mew-decode-message
 	       ;; Call internalform with VIRTUAL content header
 	       ;;     CT: message/rfc822 (virtual)
-	       ;; 
+	       ;;
 	       ;;     Header(RFC822 header + content header)
 	       ;;
 	       ;;     Body(content body)
@@ -423,7 +450,7 @@ Return a part syntax after moving the beginning of the content body."
   ;; Called on the beginning of the RFC822 header in the narrowed region
   ;; hbeg is certainly the beginning of the VIRTUAL content body(i.e. min).
   ;; hend will have to set to the end of PHYSICAL content header(i.e. end)
-  ;; after analyzing the physical content header and body since CD:'s 
+  ;; after analyzing the physical content header and body since CD:'s
   ;; length in the physical content header will change(no need to say
   ;; about the end of the physical content header).
   ;;
@@ -458,12 +485,12 @@ Return a part syntax after moving the beginning of the content body."
 	   (mew-inherit-decode-signer (nth 2 msf)) ;; signature verification
 	   part)
       ;; the beginning of the physical content header (cur)
-      (cond 
+      (cond
        (mimep ;; MIME
 	(save-restriction
 	  (narrow-to-region (point) (point-max))
 	  (setq part (mew-decode-singlepart cnt nil 'message))
-	  ;; hend is always 1 char smaller than the beginning of 
+	  ;; hend is always 1 char smaller than the beginning of
 	  ;; the physical content body.
 	  ;; If multipart/alternative mew-syntax-get-begin2 gets
 	  ;; correct value like other cases.
@@ -499,9 +526,8 @@ Return a part syntax after moving the beginning of the content body."
     (unless (or (null cte) (mew-cte-composite-p cte))
       (when (and (mew-case-equal cte mew-b64) (fboundp 'base64-decode-region))
 	(condition-case nil
-	    ;; Emacs 20
 	    (setq decoded (base64-decode-region beg (point-max)))
-	  (error (setq decoded nil))) ;; Emacs 21
+	  (error (setq decoded nil)))
 	(when (and decoded linebasep)
 	  (goto-char beg)
 	  (mew-crlf-to-lf)))
@@ -519,16 +545,14 @@ Return a part syntax after moving the beginning of the content body."
 	    ;; Never reach here when decoding.
 	    (mew-decode-error (concat "Unknown CTE: " cte))
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy mew-cs-text-for-write
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region beg (point-max) file nil 'no-msg))
+	  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region beg (point-max) file nil 'no-msg))
 	  (delete-region beg (point-max))
-	  (mew-piolet
-	   mew-cs-binary mew-cs-dummy
-	   ;; mew-prog-mime-decode converts CRLF to LF, so
-	   ;; read input as binary.
-	   (apply 'call-process mew-prog-mime-decode file '(t nil) nil opt))
+	  (mew-piolet mew-cs-binary mew-cs-dummy
+	    ;; mew-prog-mime-decode converts CRLF to LF, so
+	    ;; read input as binary.
+	    (apply 'call-process mew-prog-mime-decode file '(t nil) nil opt))
 	  (mew-delete-file file)))
        (t
 	(mew-decode-error (concat mew-prog-mime-decode " does not exist")))))
@@ -575,7 +599,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (ct (mew-syntax-get-value ctl 'cap))
 	 (textp (mew-ct-textp ct))
 	 (cte (or (mew-syntax-get-cte syntax) mew-7bit))
-	 (multi-err t) (encap nil) type)
+	 (multi-err t) (encap nil) type hend format delsp)
     ;; the beginning of the content body
     (cond
      ((not (mew-cte-p cte))
@@ -606,6 +630,10 @@ Return a part syntax after moving the beginning of the content body."
 	  ;; xxx how about message/partial?
 	  (mew-syntax-set-ct syntax mew-type-apo)
 	  ))))
+     ((or (string= ct mew-ct-smm) (string= ct mew-ct-xsmm))
+      (mew-decode-mime-body nil ct cte)
+      (mew-syntax-set-end syntax (point-max))
+      (setq syntax (mew-decode-smime syntax cnt)))
      ;; Multipart, decoding is not required
      ((mew-ct-multipartp ct)
       (if (mew-cte-composite-p cte)
@@ -621,6 +649,7 @@ Return a part syntax after moving the beginning of the content body."
 	 ((string= mew-ct-mls ct)
 	  (setq syntax (mew-decode-multipart-signed syntax cnt)))
 	 ((string= mew-ct-mle ct)
+	  ;; xxx making use of password cache?
 	  (if mew-inherit-prefetching
 	      (signal 'quit "")
 	    (setq syntax (mew-decode-multipart-encrypted syntax cnt))))
@@ -635,7 +664,10 @@ Return a part syntax after moving the beginning of the content body."
 	(mew-decode-rfc822-header)))
      (t
       (if (and (eq parent 'message)
-	       (or (not (mew-xinfo-get-text-body)) (not textp)))
+	       (or (not (mew-xinfo-get-text-body))
+		   (if (mew-dinfo-get-encap-html) 
+		       (not (string= ct mew-ct-txt))
+		     (not textp))))
 	  (setq encap t))
       ;; even if cte is nil, call mew-decode-mime-body for charset conversion
       (if textp
@@ -648,13 +680,24 @@ Return a part syntax after moving the beginning of the content body."
     ;; or single in multipart/alternative
     ;; or single in multipart/security
     (setq type (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-    (unless (mew-syntax-get-end syntax)
+    (setq hend (mew-syntax-get-end syntax))
+    (unless hend
       (mew-syntax-set-end syntax (point-max)))
     (when (and (string= type mew-ct-txt)
-	       ;; highlight in the cache buffers only
+	       (setq format (mew-syntax-get-param ctl "format"))
+	       (mew-case-equal format "flowed"))
+      (setq delsp (mew-syntax-get-param ctl "delsp"))
+      (if (and delsp (mew-case-equal delsp "yes"))
+	  (setq delsp t)
+	(setq delsp nil))
+      (mew-decode-flowed (mew-syntax-get-begin syntax) (point-max) delsp)
+      (setq hend nil))
+    ;; highlight in the cache buffers only
+    (when (and (string= type mew-ct-txt)
 	       (string-match mew-buffer-cache-prefix (buffer-name)))
-      (mew-highlight-body-region (mew-syntax-get-begin syntax)
-				 (mew-syntax-get-end syntax)))
+      (mew-highlight-body-region (mew-syntax-get-begin syntax) (point-max)))
+    (unless hend
+      (mew-syntax-set-end syntax (point-max))) ;; ajusting
     (if encap
 	;; Mew allows text/plain and multipart/* for body.
 	;; If other CT: is embedded under message, it should be
@@ -668,6 +711,63 @@ Return a part syntax after moving the beginning of the content body."
       ;; return value
       syntax)))
 
+(defun mew-decode-flowed-level ()
+  (cond
+   ((looking-at "^$") -1)
+   ((looking-at "^>*")
+    (let ((level (- (match-end 0) (match-beginning 0))))
+      (goto-char (match-end 0))
+      ;; we don't remove space-stuff after >
+      (if (and (/= level 0) (looking-at " ")) (forward-char))
+      level))))
+
+(defconst mew-flowed-quoted  ?>)
+(defconst mew-flowed-stuffed mew-sp)
+(defconst mew-flowed-break   mew-sp)
+(defconst mew-flowed-netnews "-- ")
+(defvar mew-flowed-broken-list '("----- Original Message ----- ")) ;; OE
+
+(defun mew-decode-flowed (beg end delsp)
+  "Decoding wrapped lines encoded with RFC 3676"
+  (save-restriction
+    (narrow-to-region beg end)
+    (goto-char (point-min))
+    (let (level prev-level softbreak)
+      (setq level (mew-decode-flowed-level))
+      (if (= level 0) (mew-decode-flowed-remove-stuffed))
+      (setq softbreak (mew-decode-flowed-soft-breakp delsp))
+      (while (= (forward-line) 0)
+	(setq prev-level level)
+	(setq level (mew-decode-flowed-level))
+	(cond
+	 ((and (= prev-level level) softbreak)
+	  (mew-decode-flowed-remove-quoted-stuffed softbreak))
+	 (t 
+	  (if (= level 0) (mew-decode-flowed-remove-stuffed))))
+	(setq softbreak (mew-decode-flowed-soft-breakp delsp))))))
+
+(defun mew-decode-flowed-remove-quoted-stuffed (beg)
+  (delete-region beg (point)))
+
+(defun mew-decode-flowed-remove-stuffed ()
+  (if (and (char-after) (char-equal (char-after) mew-flowed-stuffed))
+      (delete-char 1)))
+
+(defun mew-decode-flowed-soft-breakp (delsp)
+  (let ((beg (point)) line)
+    (end-of-line)
+    (setq line (mew-buffer-substring beg (point)))
+    (if (string= line mew-flowed-netnews)
+	nil
+      (if (and mew-decode-broken (member line mew-flowed-broken-list))
+	  (progn
+	    (mew-xinfo-set-warning
+	     (cons "Illegal flowed lines\n" (mew-xinfo-get-warning)))
+	    nil)
+	(if (and (char-before) (char-equal (char-before) mew-flowed-break))
+	    (if delsp (1- (point)) (point))
+	  nil)))))
+    
 ;;;
 ;;; Decoding multipart
 ;;;
@@ -675,15 +775,103 @@ Return a part syntax after moving the beginning of the content body."
 (defun mew-decode-multipart-boundary-regex (boundary)
   ;; for broken MUAs which uses non-ASCII boundaries
   ;; This works only on Emacs 21.
-  (setq boundary (mew-set-string-multibyte boundary))
-  (concat "^--" (regexp-quote boundary) "\\(--\\|\\)[ \t]*$"))
+  (let ((mboundary (mew-set-string-multibyte boundary)))
+    ;; Some MUAs use invalid character like "<>".
+    (when (or (string-match "[!-&*;<>@[-^`{-~]" mboundary)
+	      (not (string= mboundary boundary)))
+      (if mew-decode-broken
+	  (mew-decode-warning-params "boundary" 'char)
+	(mew-decode-error "Boundary has invalid character")))
+    (concat "^--" (regexp-quote mboundary) "\\(--\\|\\)[ \t]*$")))
 
-(defsubst mew-decode-multipart-boundary-cont ()
+(defun mew-decode-multipart-end-boundary-regex (boundary)
+  (let ((mboundary (mew-set-string-multibyte boundary)))  
+    (concat "^--" (regexp-quote mboundary) "--[ \t]*$")))
+
+(defun mew-decode-multipart-boundary-cont ()
   (string= (mew-match-string 1) ""))
 
-(defsubst mew-decode-multipart-boundary-end ()
+(defun mew-decode-multipart-boundary-end ()
   (string= (mew-match-string 1) "--"))
 
+(defun mew-decode-multipart-beginning-of-first-part (bregex ct)
+  ;; RFC 2046 says, "An exact match of the entire candidate
+  ;; line is not required; it is sufficient that the boundary
+  ;; appear in its entirety following the CRLF".
+  (unless (and (re-search-forward bregex nil t)
+	       (mew-decode-multipart-boundary-cont))
+    (mew-decode-error (format "No first boundary for %s" ct)))
+  (forward-line)) ;; the beginning of the first part
+
+(defun mew-decode-multipart-beginning-of-part (break)
+  (if break
+      () ;; close-delimiter need not be followed by CRLF
+    (forward-line))) ;; the beginning of the next part
+
+(defun mew-decode-multipart-end-of-part (break)
+  (unless break (forward-line -1))
+  (beginning-of-line)
+  (forward-char -1) ;; skip the preceding CRLF
+  ;; the end of the part
+  )
+
+(defun mew-decode-multipart-singlepart (break start cnt dct)
+  (save-excursion
+    (mew-decode-multipart-end-of-part break)
+    (save-restriction
+      (narrow-to-region start (point))
+      (goto-char (point-min))
+      ;; the beginning of the part
+      (mew-decode-singlepart cnt dct nil))))
+
+(defun mew-decode-multipart-duplicate-boundary (start cnt dct err boundary bregex)
+  (if (not (string-match "^No first boundary" (nth 1 err)))
+      (error "")
+    ;; duplicate boundary error recovering, sigh
+    (widen) ;; breaking save-restriction
+    (goto-char start) ;; breaking save-excursion
+    (let ((eregex (mew-decode-multipart-end-boundary-regex boundary))
+	  break part)
+      (if (not (and (re-search-forward eregex nil t)
+		    (forward-line)
+		    (re-search-forward bregex nil t)))
+	  (error "")
+	(mew-xinfo-set-warning
+	 (cons "Duplicate boundary\n" (mew-xinfo-get-warning)))
+	(mew-xinfo-set-decode-err nil)
+	(setq break (mew-decode-multipart-boundary-end))
+	(mew-decode-multipart-beginning-of-part break)
+	(setq part (mew-decode-multipart-singlepart break start cnt dct))
+	(list part break)))))
+
+(defun mew-decode-multipart-alternative-choice (part prefpart lastpref lastatpref)
+  (let ((pref (or (mew-multipart-alternative-preference part)
+		  (length mew-mime-multipart-alternative-list)))
+	(atpref (mew-mime-external-body-preference part)))
+    ;; returns '(prefpart lastpref lastatpref)
+    (cond
+     ((or (null prefpart) (< pref lastpref))
+      (list part pref atpref))
+     ((and (= pref lastpref) atpref
+	   (or (null lastatpref) (< atpref lastatpref)))
+      ;; If external-body and internal-body are alternative,
+      ;; what should we do?
+      (list part lastpref atpref))
+     (t
+      (list prefpart lastpref lastatpref)))))
+
+(defun mew-decode-multipart-alternative-part (prefpart syntax)
+  (let* ((beg (mew-syntax-get-begin prefpart))
+	 (ctl (mew-syntax-get-ct prefpart))
+	 (ct (mew-syntax-get-value ctl 'cap))
+	 (multibeg (mew-syntax-get-begin syntax)))
+    (mew-syntax-set-begin prefpart (cons beg multibeg))
+    (mew-xinfo-set-info
+     (cons
+      (format "%s in Multipart/Alternative as a singlepart\n" ct)
+      (mew-xinfo-get-info)))
+    prefpart))
+
 (defun mew-decode-multipart (syntax cnt &optional dct)
   (let* ((case-fold-search nil) ;; boundary is case sensitive
 	 (ctl (mew-syntax-get-ct syntax))
@@ -691,67 +879,34 @@ Return a part syntax after moving the beginning of the content body."
 	 (boundary (mew-syntax-get-param ctl "boundary"))
 	 (count 0)
 	 (parts []) part
-	 pref lastpref prefpart atpref lastatpref
+	 (use-alt (and (mew-dinfo-get-use-alt) (string= ct mew-ct-mla)))
+	 prefpart lastpref lastatpref
 	 bregex start break)
     (unless boundary
       (mew-decode-error "No boundary parameter for multipart"))
     (mew-syntax-set-key syntax 'multi)
-    ;; RFC 2046 says, "An exact match of the entire candidate 
-    ;; line is not required; it is sufficient that the boundary 
-    ;; appear in its entirety following the CRLF".
     (setq bregex (mew-decode-multipart-boundary-regex boundary))
-    (unless (and (re-search-forward bregex nil t)
-		 (mew-decode-multipart-boundary-cont))
-      (mew-decode-error (format "No first boundary for %s" ct)))
-    (forward-line) ;; the beginning of the next part
-    (setq start (point)) ;; the beginning of the part
+    (mew-decode-multipart-beginning-of-first-part bregex ct)
+    (setq start (point)) ;; the beginning of the first part
     (catch 'multipart
       (while (re-search-forward bregex nil t)
 	(setq break (mew-decode-multipart-boundary-end))
-	(if break
-	    () ;; close-delimiter need not be followed by CRLF
-	  (forward-line)) ;; the beginning of the next part
-	(save-excursion
-	  (unless break (forward-line -1))
-	  (beginning-of-line)
-	  (forward-char -1) ;; skip the preceding CRLF
-	  ;; the end of the part
-	  (save-restriction
-	    (narrow-to-region start (point))
-	    (goto-char (point-min))
-	    ;; the beginning of the part
-	    (setq part (mew-decode-singlepart cnt dct nil))
-	    (setq count (1+ count))
-	    (if (and (mew-dinfo-get-use-alt) (string= ct mew-ct-mla))
-		(progn
-		  (setq pref (or (mew-multipart-alternative-preference part)
-				 (length mew-mime-multipart-alternative-list)))
-		  (setq atpref (mew-mime-external-body-preference part))
-		  (cond
-		   ((or (null prefpart) (< pref lastpref))
-		    (setq prefpart part)
-		    (setq lastpref pref)
-		    (setq lastatpref atpref))
-		   ((and (= pref lastpref) atpref)
-		    ;; If external-body and internal-body are alternative,
-		    ;; what should we do?
-		    (when (or (null lastatpref) (< atpref lastatpref))
-		      (setq prefpart part)
-		      (setq lastatpref atpref)))))
-	      (setq parts (vconcat parts (vector part))))))
+	(mew-decode-multipart-beginning-of-part break)
+	(condition-case err
+	    (setq part (mew-decode-multipart-singlepart break start cnt dct))
+	  (error
+	   ;; rescue duplicate boundary
+	   (let ((pb (mew-decode-multipart-duplicate-boundary start cnt dct err boundary bregex)))
+	     (mew-set '(part break) pb))))
+	(setq count (1+ count))
 	(setq start (point)) ;; the beginning of the part
+	(if use-alt
+	    (let ((pll (mew-decode-multipart-alternative-choice part prefpart lastpref lastatpref)))
+	      (mew-set '(prefpart lastpref lastatpref) pll))
+	  (setq parts (vconcat parts (vector part))))
 	(when break
-	  (if (and (mew-dinfo-get-use-alt) (string= ct mew-ct-mla))
-	      (let* ((beg (mew-syntax-get-begin prefpart))
-		     (ctl (mew-syntax-get-ct prefpart))
-		     (ct (mew-syntax-get-value ctl 'cap))
-		     (multibeg (mew-syntax-get-begin syntax)))
-		(mew-syntax-set-begin prefpart (cons beg multibeg))
-		(mew-xinfo-set-info
-		 (cons
-		  (format "%s in Multipart/Alternative as a singlepart\n" ct)
-		  (mew-xinfo-get-info)))
-		(throw 'multipart prefpart))
+	  (if use-alt
+	      (throw 'multipart (mew-decode-multipart-alternative-part prefpart syntax))
 	    (throw 'multipart (vconcat syntax parts)))))
       ;; Let's try to decode the first part even if boundary is not found.
       ;; This should save truncated messages.
@@ -766,6 +921,8 @@ Return a part syntax after moving the beginning of the content body."
 ;;; Privacy services
 ;;;
 
+(mew-defstruct privacy-dinfo ct proto result)
+
 (defun mew-decode-multipart-encrypted (syntax cnt)
   ;; called in narrowed region
   ;;
@@ -804,7 +961,7 @@ Return a part syntax after moving the beginning of the content body."
 	(if existp (setq file1 (mew-save-decode-form syntax1)))
       (setq unknown t))
     (forward-line) ;; the beginning of the encrypted part
-    (setq start (point)) 
+    (setq start (point))
     ;;
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-end))
@@ -815,7 +972,7 @@ Return a part syntax after moving the beginning of the content body."
 		     (mew-decode-security-singlepart start (1- (point))))))
     ;;
     (delete-region (point-min) (point-max))
-    ;; 
+    ;;
     ;; Call protocol function
     (cond
      (unknown
@@ -828,7 +985,7 @@ Return a part syntax after moving the beginning of the content body."
       (mew-set '(file3 result) file3result)))
     ;;
     (if (and func existp (file-exists-p file3))
-	(mew-flet 
+	(mew-flet
 	 (mew-insert-file-contents file3)
 	 (put-text-property (point-min) (point-max) 'mew-noncontents nil)
 	 ;; because of RICH functionality of RFC1847... Gee dirty!
@@ -846,7 +1003,7 @@ Return a part syntax after moving the beginning of the content body."
     (setq privacy (mew-syntax-get-privacy syntax3))
     (if privacy (setq result (concat result "\n\t")))
     (mew-syntax-set-privacy
-     syntax3 (cons (list mew-ct-mle proto result) privacy))
+     syntax3 (cons (mew-make-privacy-dinfo :ct mew-ct-mle :proto proto :result result) privacy))
     syntax3))
 
 (defun mew-decode-multipart-signed (syntax cnt)
@@ -879,10 +1036,10 @@ Return a part syntax after moving the beginning of the content body."
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-cont))
       (mew-decode-error "No second boundary for Multipart/Signed"))
-    (beginning-of-line) 
+    (beginning-of-line)
     (setq end1 (1- (point))) ;; the end of the signed part
     (forward-line) ;; the beginning of the key part
-    (setq start2 (point)) 
+    (setq start2 (point))
     ;;
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-end))
@@ -918,7 +1075,7 @@ Return a part syntax after moving the beginning of the content body."
     (setq privacy (mew-syntax-get-privacy syntax3))
     (if privacy (setq result (concat result "\n\t")))
     (mew-syntax-set-privacy
-     syntax3 (cons (list mew-ct-mls proto result) privacy))
+     syntax3 (cons (mew-make-privacy-dinfo :ct mew-ct-mls :proto proto :result result) privacy))
     syntax3))
 
 (defun mew-decode-security-singlepart (beg end)
@@ -943,13 +1100,13 @@ Return a part syntax after moving the beginning of the content body."
     (save-excursion
       (goto-char (point-min))
       (catch 'header
-	(while (re-search-forward 
+	(while (re-search-forward
 		"^\r?$\\|^Content-Transfer-Encoding:[ \t]*" nil t)
 	  (setq key (mew-match-string 0))
 	  (setq start (match-end 0))
 	  (when (string-match "^\r?$" key)
 	    (save-restriction
-	      (if (string-match mew-bin cte)
+	      (if (string-match cte mew-bin)
 		  (narrow-to-region (point-min) (1+ start))
 		(narrow-to-region (point-min) (point-max)))
 	      (goto-char (point-min))
@@ -964,13 +1121,13 @@ Return a part syntax after moving the beginning of the content body."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -979,7 +1136,7 @@ Return a part syntax after moving the beginning of the content body."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
