@@ -67,7 +67,7 @@
 \\{mew-draft-mode-map}"
   (interactive)
   (setq major-mode 'mew-draft-mode)
-  (setq mode-line-buffer-identification mew-mode-line-id)
+  (setq mode-line-buffer-identification (mew-mode-line-id))
   (mew-draft-set-local-variables)
   (use-local-map mew-draft-mode-map)
   (set-syntax-table mew-draft-mode-syntax-table)
@@ -571,7 +571,7 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
     (setq prefix (or nickname user))
     (if mew-ask-cite-prefix
 	(setq prefix (read-string "Citation prefix: " prefix)))
-    (concat prefix mew-cite-prefix)))
+    (concat prefix mew-cite-default-prefix)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -765,6 +765,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
       (mew-draft-process-message 'send))))
 
 (defun mew-draft-process-message (action &optional privacy signer)
+  (if (and (boundp 'longlines-mode) longlines-mode) (longlines-mode -1))
   (run-hooks 'mew-make-message-hook)
   (let* ((case (or (mew-tinfo-get-case) mew-case-default))
 	 (old-case case)
@@ -974,7 +975,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
