@@ -12,10 +12,10 @@
 ;;; Suppressing warnings at compile
 ;;;
 
-(defmacro mew-no-waring-defvar (var-name)
+(defmacro mew-no-warning-defvar (var-name)
   `(unless (boundp ',var-name) (defvar ,var-name nil)))
 
-(defmacro mew-no-waring-defun (func-name)
+(defmacro mew-no-warning-defun (func-name)
   `(unless (fboundp ',func-name) (defun ,func-name(&rest args) nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -45,7 +45,8 @@ requires PTY.")
 (defvar mew-icon-p (featurep 'tool-bar))
 
 (defvar mew-internal-utf-8p nil)
-(if (fboundp 'utf-translate-cjk-mode) ;; Emacs 22.1 or later
+(if (or (fboundp 'utf-translate-cjk-mode) ;; Emacs 22.1 or later
+	(coding-system-p 'utf-8-emacs))
     (setq mew-internal-utf-8p t))
 
 (require 'mew-key)
