@@ -272,46 +272,41 @@
 	 (prog (mew-image-get-prog ent))
 	 (prog2 (mew-image-get-prog2 ent))
 	 (func-size (mew-image-get-func ent))
-	 image-width image-height image)
-    (when (and mew-image-display-resize func-size (fboundp func-size))
-      (save-excursion
-	(set-buffer cache)
-	(goto-char begin)
-	(setq image-width (car (funcall func-size)))
-	(setq image-height (cdr (funcall func-size)))))
-    (if (and image-width image-height
-	     (or (< width image-width)
-		 (and mew-image-display-resize-care-height (< height image-height)))
-	     (mew-which-exec prog))
-	(with-temp-buffer
-	  (mew-plet
-	   (message "Resizing image...")
-	   (insert-buffer-substring cache begin end)
-	   (mew-set-buffer-multibyte nil)
-	   (call-process-region (point-min) (point-max) prog
-				t '(t nil) nil)
-	   (if mew-image-display-resize-care-height
-	       (call-process-region (point-min) (point-max) "pnmscale"
-				    t '(t nil) nil
-				    "-xysize"
-				    (format "%d" width)
-				    (format "%d" height))
+	 image-size image-width image-height image)
+    (with-temp-buffer
+      (mew-plet
+       (insert-buffer-substring cache begin end)
+       (mew-set-buffer-multibyte nil)
+       (goto-char (point-min))
+       (setq image-size (funcall func-size))
+       (setq image-width (car image-size))
+       (setq image-height (cdr image-size))
+       (when (and image-width image-height
+		  (or (< width image-width)
+		      (and mew-image-display-resize-care-height (< height image-height)))
+		  (mew-which-exec prog))
+	 (message "Resizing image...")
+	 (call-process-region (point-min) (point-max) prog
+			      t '(t nil) nil)
+	 (if mew-image-display-resize-care-height
 	     (call-process-region (point-min) (point-max) "pnmscale"
 				  t '(t nil) nil
-				  "-xsize" (format "%d" width)))
-	   (if (and (string< emacs-version "22") ;; xxx
-		    (mew-which-exec prog2))
-	       (call-process-region (point-min) (point-max) prog2
-				    t '(t nil) nil)
-	     (setq format 'pbm))
-	   (setq image (mew-buffer-substring (point-min) (point-max)))
-	   (message "Resizing image...done")))
-      (save-excursion
-	(set-buffer cache)
-	(setq image (mew-buffer-substring begin end))))
+				  "-xysize"
+				  (format "%d" width)
+				  (format "%d" height))
+	   (call-process-region (point-min) (point-max) "pnmscale"
+				t '(t nil) nil
+				"-xsize" (format "%d" width)))
+	 (if (and (string< emacs-version "22") ;; xxx
+		  (mew-which-exec prog2))
+	     (call-process-region (point-min) (point-max) prog2
+				  t '(t nil) nil)
+	   (setq format 'pbm))
+	 (message "Resizing image...done"))
+       (setq image (mew-buffer-substring (point-min) (point-max)))))
     (mew-elet
      (condition-case nil
-	 (insert-image (create-image (string-as-unibyte image) format t))
+	 (insert-image (create-image image format t))
        (error ()))))
   (goto-char (point-min))
   (message "Loading image...done"))
@@ -374,7 +369,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2008 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
