@@ -359,7 +359,13 @@
 	    ;; broken quoted string cannot be rescued because
 	    ;; the separator cannot be distinguished
 	    (if (> tmp-cnt 0) (funcall do-cons-ret))
-	    (funcall do-clear))))
+	    (funcall do-clear)))
+	 (do-rescue
+	  (lambda (var)
+	    (save-match-data
+	      (if (and addrp (string-match "<\\([^>]+\\)>" var))
+		  (setq ret (cons (mew-match-string 1 var) ret))
+		(funcall do-cons-ret))))))
     ;; main
     (catch 'max
       (while (< i len)
@@ -379,12 +385,12 @@
 	 (t
 	  (funcall do-copy)))
 	(setq i (1+ i)))
-      (when (> tmp-cnt 0)
-	;; broken quoted string can be rescued if it appears solely
-	(save-match-data
-	  (if (and addrp (string-match "<\\([^>]+\\)>" tmp))
-	      (setq ret (cons (mew-match-string 1 tmp) ret))
-	    (funcall do-cons-ret)))))
+      ;; broken quoted string can be rescued if it appears solely
+      (cond
+       ((> par-cnt 0)
+	(funcall do-rescue str))
+       ((> tmp-cnt 0)
+	(funcall do-rescue tmp))))
     (setq ret (delete nil ret))
     (when allow-spc
       (setq ret (mapcar (lambda (str)
@@ -510,7 +516,7 @@ Remove anonymous addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
