@@ -131,7 +131,7 @@
 	  (setq warnp nil))
 	 ((member domain (mew-warn-domains case))
 	  (setq warnp t))
-	 ((or (mew-safe-addresses case) (mew-safe-domains case))
+	 ((or (mew-warn-addresses case) (mew-safe-domains case))
 	  (setq warnp t))
 	 (t
 	  (setq warnp nil)))
@@ -355,7 +355,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2008 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
