@@ -485,7 +485,7 @@
 (defun mew-imap-command-wmbx (pro pnm)
   (let* ((rtr (mew-imap-get-rfl pnm))
 	 (dst (mew-copyinfo-get-dst rtr)))
-    (mew-imap-set-error pnm (format "Ccannot create '%s' or the messages do not exist" (mew-imap-escape-format dst)))
+    (mew-imap-set-error pnm (format "Cannot create '%s' or the messages do not exist" (mew-imap-escape-format dst)))
     (mew-imap-set-status pnm "logout2")
     (mew-imap-command-logout2 pro pnm)))
 
@@ -1220,8 +1220,16 @@
 (defun mew-imap-make-mark-hist (ents)
   (let (ret)
     (dolist (ent ents)
-      (when (memq (mew-mark-hist-get-mark ent) mew-imap-marks-to-sync)
-	(setq ret (cons ent ret))))
+      (let ((mark (mew-mark-hist-get-mark ent))
+	    (msg (mew-mark-hist-get-msg ent)))
+	(cond
+	 ((memq mark mew-imap-marks-to-sync)
+	  (setq ret (cons ent ret)))
+	 ;; xxx the refile mark overrides the read mark.
+	 ;; If we refile a message, we cannot distinguish
+	 ;; read or unread...
+	 ((char-equal mark mew-mark-refile)
+	  (setq ret (cons (mew-make-mark-hist :msg msg :mark mew-mark-read) ret))))))
     (nreverse ret)))
 
 (defun mew-imap-retrieve (case directive bnm &rest args)
@@ -1845,7 +1853,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2008 Mew developing team.
+;; Copyright (C) 2002-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
