@@ -28,7 +28,7 @@
   (interactive)
   (setq major-mode 'mew-message-mode)
   (setq mode-name mew-mode-name-message)
-  (setq mode-line-buffer-identification mew-mode-line-id)
+  (setq mode-line-buffer-identification (mew-mode-line-id))
   (use-local-map mew-message-mode-map)
   (setq buffer-read-only t)
   (make-local-variable 'page-delimiter)
