@@ -7,8 +7,8 @@
 
 (require 'mew)
 (eval-when-compile
-  (mew-no-waring-defun w3m-region)
-  (mew-no-waring-defun w3m-expand-file-name-as-url))
+  (mew-no-warning-defun w3m-region)
+  (mew-no-warning-defun w3m-expand-file-name-as-url))
 
 (defvar mew-process-file-alist nil)
 
