@@ -34,7 +34,7 @@ This function are used to check validity of 'case' and sort key.")
                     (fboundp mew-input-exit-minibuffer-function)))
           (funcall mew-input-exit-minibuffer-function))
       (exit-minibuffer)))
- 
+
 (defvar mew-input-comma-function nil
   "A function to be called when ',' is typed in minibuffer.
 This function can be used to check validity of 'case'.")
@@ -107,10 +107,7 @@ at completions."
 	   (goto-char (match-beginning 1))
 	   (setq face (if (not window-system)
 			  'mew-face-header-from
-			(if (or (and mew-xemacs-p (find-face 'isearch))
-				(and (not mew-xemacs-p) (facep 'isearch)))
-			    'isearch
-			  'region)))
+			(if (facep 'isearch) 'isearch 'region)))
 	   (put-text-property (point) (match-end 1) 'face face)
 	   (unless (pos-visible-in-window-p (point) win)
 	     (set-window-start win (progn (forward-line -2) (point))))))))))
@@ -135,7 +132,7 @@ at completions."
     (setq buffer-read-only t)
     (mew-elet
      (mew-erase-buffer)
-     (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
+     (mapc (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
     (if min (goto-char (point-min)))))
 
 (defun mew-input-folder-search-setup1 (&optional min)
@@ -173,7 +170,7 @@ at completions."
   (let (do-search alist)
     (save-excursion
       (goto-char (point-max))
-      (if (search-backward "," nil t)
+      (if (search-backward "," (minibuffer-prompt-end) t)
 	  (progn
 	    (goto-char (match-end 0))
 	    (unless (looking-at mew-regex-case2)
@@ -219,7 +216,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -260,7 +257,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -334,20 +331,12 @@ and does not insert anything on single folder input."
 When in folder search mode, this function searches a candidate
 folder and displays it in addition to its bound key."
   (interactive)
-  (let* ((key (if mew-xemacs-p
-		  (event-key last-command-event)
-		(this-command-keys)))
-	 last-str gfunc)
+  (let ((key (this-command-keys))
+	last-str gfunc)
     (cond
-     ((mew-characterp key) ;; XEmacs
-      (setq key (char-to-string key))
-      (setq last-str key)
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((stringp key)
       (setq last-str key)
       (setq gfunc (lookup-key (current-global-map) key)))
-     ((symbolp key) ;; XEmacs
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((vectorp key)
       (setq gfunc (lookup-key (current-global-map) key))
       (unless gfunc
@@ -391,7 +380,7 @@ folder and displays it in addition to its bound key."
        ((null gfunc)
 	())
        ((string-match "self-insert-command" (symbol-name gfunc))
-	(insert last-command-char))
+	(insert last-command-event))
        ((and (fboundp gfunc) (commandp gfunc))
 	(call-interactively gfunc))))))
 
@@ -400,17 +389,16 @@ folder and displays it in addition to its bound key."
 ;;; Prefix hack
 ;;;
 
-(defun mew-input-folder-complete-case-input (prefix-char)
-  (when (and mew-use-case-input-completion
+(defun mew-input-folder-complete-case (prefix-char)
+  (when (and mew-use-case-completion
 	     (eq mew-input-complete-function 'mew-complete-folder)
 	     (mew-folder-remotep (char-to-string prefix-char))
-	     (and mew-case-input
-		  (not (string= mew-case-input mew-case-default))))
-    (insert mew-case-input ":")))
+	     (and mew-case (not (string= mew-case mew-case-default))))
+    (insert mew-case ":")))
 
 (defun mew-input-folder-prefix ()
   "A function to insert a folder prefix in minibuffer.
-If the previous character is another folder prefix, 
+If the previous character is another folder prefix,
 it is deleted automatically."
   (interactive)
   (cond
@@ -424,55 +412,55 @@ it is deleted automatically."
 	   (prefix (if (eq mew-input-complete-function
 			   'mew-complete-local-folder)
 		       mew-folder-local mew-folder-imap))
-	   (insert-ok (string= (char-to-string last-command-char) prefix)))
+	   (insert-ok (string= (char-to-string last-command-event) prefix)))
       (cond
        ((or (= pos 0)
 	    (save-excursion (forward-char -1) (looking-at ",")))
 	(if insert-ok (insert prefix)))
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
-	;; A wrong prefix might accidentally be here. 
+	;; A wrong prefix might accidentally be here.
 	;; So, replace it just in case
 	(when insert-ok
 	  (forward-char -1)
 	  (delete-char 1)
 	  (insert prefix)))
        (t
-	(insert last-command-char)))))
+	(insert last-command-event)))))
    (t
     (let ((pos (- (point) (mew-minibuf-point-min))))
       (cond
        ((or (= pos 0)
 	    (save-excursion (forward-char -1) (looking-at ",")))
-	(mew-input-folder-complete-case-input last-command-char)
-	(insert last-command-char))
+	(mew-input-folder-complete-case last-command-event)
+	(insert last-command-event))
        ;;
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
 	(forward-char -1)
 	(delete-char 1)
-	(mew-input-folder-complete-case-input last-command-char)
-	(insert last-command-char))
+	(mew-input-folder-complete-case last-command-event)
+	(insert last-command-event))
        ;;
        ((save-excursion
 	  (forward-char -2)
 	  (looking-at ":[-+%$*]"))
 	(forward-char -1)
 	(delete-char 1)
-	(when (memq last-command-char '(?+ ?*))
+	(when (memq last-command-event '(?+ ?*))
 	  ;; delete case:
 	  (delete-region (save-excursion (or (and (search-backward "," nil t)
 						  (match-end 0))
 					     (mew-minibuf-point-min)))
 			 (point)))
-	(insert last-command-char))
+	(insert last-command-event))
        ;;
        (t
-	(insert last-command-char)))))))
+	(insert last-command-event)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -480,10 +468,8 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-folder-check (folders)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	(if (string-match mew-regex-case case:folder)
 	    (progn
@@ -496,9 +482,6 @@ it is deleted automatically."
 	(if (and (mew-folder-popp folder) ;; not $inbox
 		 (not (string= mew-pop-inbox-folder folder)))
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
 	(setq ret (cons case:folder ret))))
     (setq ret (delete nil ret))
@@ -530,10 +513,20 @@ it is deleted automatically."
 	 ;; mew-inherit-case must be nil
 	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
 				    init mew-input-folder-map nil
-				    'mew-input-folder-hist)))
+				    'mew-input-folder-hist))
+	 (new-folder))
     (when (or (string= ret "") (string= ret init))
       (setq ret default))
-    (car (mew-input-folder-check (list ret)))))
+    ;; mew-input-folder-check requires an IMAP separator, 
+    ;; which requires C-uZ. C-uZ needs "g". So, chicken and egg.
+    ;; Skip the check for inbox to avoid the problem.
+    (setq new-folder (mew-case:folder-folder ret))
+    (if (member new-folder (list mew-inbox-folder
+				 mew-pop-inbox-folder
+				 mew-imap-inbox-folder
+				 mew-nntp-newsgroup))
+	ret
+      (car (mew-input-folder-check (list ret))))))
 
 (defun mew-input-folders (case:folder)
   "Input a folder from the minibuffer."
@@ -553,10 +546,8 @@ it is deleted automatically."
     (mew-input-folder-check ret)))
 
 (defun mew-input-refile-folder-check (folders proto)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	;; If case is specified, just remove it at this moment...
 	(if (string-match mew-regex-case case:folder)
@@ -574,9 +565,6 @@ it is deleted automatically."
 	(if (and (= (length folder) 1)
 		 (member folder mew-folder-prefixes)) ;; "prefix" only
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq folder (mew-canonicalize-case-folder folder))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
@@ -627,7 +615,7 @@ it is deleted automatically."
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-address)
 	val vals addrs ret)
-    (setq val (read-from-minibuffer 
+    (setq val (read-from-minibuffer
 	       (if default (format prompt default) prompt)
 	       ""
 	       mew-input-map
@@ -636,9 +624,7 @@ it is deleted automatically."
     (if (and default (string= val ""))
 	(setq val default))
     (setq vals (mapcar 'mew-chop (mew-split-quoted val ?,)))
-    (while vals
-      (setq val (car vals))
-      (setq vals (cdr vals))
+    (dolist (val vals)
       (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
       (setq addrs (mapcar 'mew-addrstr-append-domain addrs))
       (setq ret (nconc ret addrs)))
@@ -651,30 +637,45 @@ it is deleted automatically."
 
 (defvar mew-input-pick-pattern-hist nil)
 
-(defun mew-input-pick-pattern ()
+(defun mew-input-pick-pattern (prompt)
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-pick-pattern)
 	(mew-circular-complete-function 'mew-circular-complete-pick-pattern)
 	(keymap (copy-keymap mew-input-map)))
     (define-key keymap " " nil)
     (mew-pick-macro-expand-string
-     (read-from-minibuffer "Pick pattern: "
+     (read-from-minibuffer (concat prompt " pattern: ")
 			   (car mew-pick-pattern-list)
 			   keymap
 			   nil
 			   'mew-input-pick-pattern-hist))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Grep pattern
-;;;
+(defvar mew-input-pick-command-hist nil)
 
-(defvar mew-input-grep-pattern-hist nil)
-
-(defun mew-input-grep-pattern ()
-  (read-string "Grep pattern: "
-               (car-safe mew-input-grep-pattern-hist)
-               'mew-input-grep-pattern-hist))
+(defun mew-input-pick-command (prog opts)
+  (mew-input-clear)
+  (let ((init (mapconcat 'identity (cons prog opts) " "))
+	(prompt "Cmd opts pattern: ")
+	input lst cmd opts len pat)
+    (unless (string-match " $" init)
+      (setq init (concat init " ")))
+    (setq input (read-string prompt init 'mew-input-pick-command-hist))
+    (setq mew-input-pick-command-hist
+	  (mew-uniq-list mew-input-pick-command-hist))
+    (setq lst (mew-split-quoted input mew-sp))
+    (setq cmd (car lst))
+    (setq opts (cdr lst))
+    (setq len (length opts))
+    (cond
+     ((= len 0)
+      )
+     ((= len 1)
+      (setq pat (car opts))
+      (setq opts nil))
+     (t
+      (setq pat (nth (1- len) opts))
+      (setcdr (nthcdr (- len 2) opts) nil)))
+    (list cmd opts pat)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -705,16 +706,16 @@ it is deleted automatically."
 		      (format "Sort by (%s)?: " key)
 		      ""
 		      mew-input-map
-		      nil 
+		      nil
 		      'mew-input-sort-key-hist))
 	 field mode)
     (if (or (null field:mode) (string= field:mode ""))
 	(setq field:mode key))
     (setq field (car (mew-split field:mode ?:)))
-    (setq mode  (or (car (cdr (mew-split field:mode ?:)))
-		    (cdr (assoc field mew-sort-key-alist))
-		    "text"))
-    (cons field mode)))
+    (setq mode (or (car (cdr (mew-split field:mode ?:)))
+		   (mew-alist-get-value (assoc field mew-sort-key-alist))
+		   "text"))
+    (list field mode)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -747,8 +748,8 @@ it is deleted automatically."
 	  mew-range-all ;; buffer is empty.
 	(forward-line -1)
 	(mew-summary-goto-message)
-	(concat (int-to-string
-		 (1+ (string-to-int (mew-summary-message-number))))
+	(concat (number-to-string
+		 (1+ (string-to-number (mew-summary-message-number))))
 		"-")))))
 
 (defun mew-input-range (folder askp)
@@ -795,7 +796,7 @@ it is deleted automatically."
      ((string= range mew-range-str-all)      t)
      ((string= range mew-range-str-update) nil)
      ((string-match "^last:\\([0-9]+\\)$" range)
-      (string-to-int (match-string 1 range)))
+      (string-to-number (match-string 1 range)))
      (t nil)))) ;; update just in case
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -862,7 +863,13 @@ it is deleted automatically."
        (setq def (concat dir ,file)))
      (setq ret (mew-input-file-name ,prompt def))
      (if ,preservep
-	 (setq ,previous (file-name-directory (mew-convert-to-home-dir ret))))
+	 (setq ,previous (file-name-directory
+			  (mew-convert-to-home-dir
+			   (if (file-directory-p ret)
+			       (file-name-as-directory ret)
+			     ret)))))
+     (if (and ,file (file-directory-p ret))
+	 (setq ret (expand-file-name (file-name-nondirectory ,file) ret)))
      ret))
 
 (defun mew-summary-input-file-name (&optional prompt file)
@@ -928,14 +935,12 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-case-check ()
-  (let* ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-         (lst (mew-split case ?,))
-	 err)
+  (let ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+	err)
     (catch 'nomatch
-      (while lst
-	(unless (member (car lst) mew-config-cases)
-	  (throw 'nomatch (setq err (car lst))))
-	(setq lst (cdr lst))))
+      (dolist (cs (mew-split case ?,))
+	(unless (member cs mew-config-cases)
+	  (throw 'nomatch (setq err cs)))))
     (if err
         (progn
           (mew-temp-minibuffer-message (format " [No match: %s]" err))
@@ -944,34 +949,32 @@ it is deleted automatically."
 
 (defvar mew-input-case-hist nil)
 
-(defun mew-input-case (default msg &optional edit)
+(defun mew-input-case (default &optional edit)
   (mew-input-clear)
   (unless default (setq default mew-case-default))
   (let ((mew-input-complete-function 'mew-complete-case)
 	(mew-circular-complete-function 'mew-circular-complete-case)
 	(mew-input-exit-minibuffer-function 'mew-input-case-check)
 	(mew-input-comma-function 'mew-input-case-check)
-	case lst ret)
+	case ret)
     (if edit
 	(setq case (read-from-minibuffer
-		    (format "%s case value: " msg)
+		    "Case value: "
 		    default
 		    mew-input-map
                     nil
                     'mew-input-case-hist))
       (setq case (read-from-minibuffer
-		  (format "%s case value (%s): " msg default)
+		  (format "Case value (%s): " default)
 		  ""
 		  mew-input-map
                   nil
                   'mew-input-case-hist)))
     (if (string= case "")
 	default
-      (setq lst (mew-split case ?,))
-      (while lst
-	(if (member (car lst) mew-config-cases)
-	    (setq ret (cons (car lst) ret)))
-	(setq lst (cdr lst)))
+      (dolist (cs (mew-split case ?,))
+	(if (member cs mew-config-cases)
+	    (setq ret (cons cs ret))))
       (mapconcat 'identity (nreverse ret) ","))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -996,106 +999,51 @@ it is deleted automatically."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Password
+;;; Encoding
 ;;;
 
-(defvar mew-passwd-alist nil)
-(defvar mew-passwd-timer-id nil)
-
-(defsubst mew-passwd-get-passwd (key)
-  (nth 1 (assoc key mew-passwd-alist)))
-(defsubst mew-passwd-get-counter (key)
-  (nth 2 (assoc key mew-passwd-alist)))
-
-(defun mew-passwd-set-passwd (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 1 (assoc key mew-passwd-alist)) val)
-    (setq mew-passwd-alist (cons (list key val 0) mew-passwd-alist))))
-(defun mew-passwd-set-counter (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 2 (assoc key mew-passwd-alist)) val)))
-
-(defun mew-passwd-get-keys ()
-  (mapcar 'car mew-passwd-alist))
-
-(defsubst mew-passwd-reset ()
-  (setq mew-passwd-alist nil))
-
-(defun mew-passwd-setup ()
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id
-	(mew-timer (* mew-passwd-timer-unit 60) 'mew-passwd-timer)))
-
-(defun mew-passwd-clean-up ()
-  (mew-passwd-reset)
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id nil))
-
-(defun mew-passwd-timer ()
-  (let ((keys (mew-passwd-get-keys)) key)
-    (while keys
-      (setq key (car keys))
-      (setq keys (cdr keys))
-      (if (< (mew-passwd-get-counter key) mew-passwd-lifetime)
-	  (mew-passwd-set-counter key (1+ (mew-passwd-get-counter key)))
-	;; time out
-	(mew-passwd-set-passwd key nil)
-	(mew-passwd-set-counter key 0)))))
-
-(defun mew-input-passwd (prompt key)
-  (if (and key mew-use-cached-passwd)
-      (if (mew-passwd-get-passwd key)
-	  (progn
-	    (mew-timing)
-	    (if mew-passwd-reset-timer (mew-passwd-set-counter key 0))
-	    (mew-passwd-get-passwd key))
-	(let ((pass (mew-read-passwd prompt)))
-	  (mew-passwd-set-passwd key pass)
-	  (mew-passwd-set-counter key 0)
-	  pass))
-    (mew-read-passwd prompt)))
-
-(defun mew-read-passwd (prompt)
-  (let ((inhibit-input-event-recording t)
-	;; A process filter sets inhibit-quit to t to prevent quitting.
-	;; Set inhibit-quit to nil so that C-g can be used
-	(inhibit-quit nil))
-    (condition-case nil
-	(read-passwd prompt)
-      ;; If read-passwd causes an error, let's return "" so that
-      ;; the password process will safely fail.
-      (quit "")
-      (error ""))))
+(defun mew-input-encoding ()
+  (let* ((encoding-list (list mew-b64 mew-qp))
+	 (encoding-alist (mapcar (lambda (x) (list x)) encoding-list))
+	 (prompt (format "Lines are too long. Input encoding (%s): "
+			 mew-default-encoding))
+	 (cte (completing-read prompt encoding-alist)))
+    (if (string= cte "") (setq cte mew-default-encoding))
+    (setq cte (downcase cte))
+    (while (not (member cte encoding-list))
+      (setq prompt (format "'%s' is unknown. Input encoding (%s): "
+			   cte mew-default-encoding))
+      (setq cte (completing-read prompt encoding-alist))
+      (if (string= cte "") (setq cte mew-default-encoding))
+      (setq cte (downcase cte)))
+    cte))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Language name
+;;; Command
 ;;;
 
-;; read-language-name should be used. Unfortunately, that of
-;; XEmacs 21.1.14 is broken. This function should be obsoleted
-;; when Mew quit support of XEmacs 21.1.14
-
-(defun mew-input-language-name (prompt &optional default) 
-  (let* ((completion-ignore-case t)
-         (name (completing-read prompt
-                                language-info-alist
-                                nil
-                                t nil nil)))
-    (if (string= name "") (setq name default))
-    name))
+(defun mew-input-command (&optional default)
+  (let ((program (read-string "Command args: " default)))
+    (setq program (mew-split-quoted program mew-sp))
+    (setq program (delete "" program))
+    (while (not (mew-which-exec (car program)))
+      (setq program (read-string "Command args: " default))
+      (setq program (mew-split-quoted program mew-sp))
+      (setq program (delete "" program)))
+    program))
 
 (provide 'mew-minibuf)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1104,7 +1052,7 @@ it is deleted automatically."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
