@@ -5,6 +5,9 @@
 
 ;;; Code:
 
+(eval-when-compile
+  (mew-no-warning-defun charset-priority-list))
+
 ;; must be here
 (if (fboundp 'find-coding-system)
     (defun mew-coding-system-p (cs)
@@ -192,7 +195,54 @@
 ;; CS
 ;;
 
-(defalias 'mew-find-cs-region 'find-charset-region)
+(defvar mew-charset-priority-list '(japanese-jisx0208
+				    korean-ksc5601
+				    latin-iso8859-2
+				    latin-iso8859-3
+				    latin-iso8859-4
+				    hebrew-iso8859-8
+				    latin-iso8859-9
+				    ;; for accent characters
+				    cyrillic-iso8859-5
+				    greek-iso8859-7
+				    latin-iso8859-14
+				    thai-tis620
+				    latin-jisx0201
+				    japanese-jisx0208-1978
+				    japanese-jisx0213-1
+				    japanese-jisx0213-2
+				    unicode))
+
+(if (fboundp 'set-charset-priority)
+    (defun mew-find-cs-region (beg end)
+      (let ((charset-list (charset-priority-list))
+	    ret lat1 lat0)
+	(catch 'find
+	  (set-charset-priority 'latin-iso8859-1)
+	  (setq ret (find-charset-region beg end))
+	  (if (equal ret '(ascii latin-iso8859-1))
+	      (throw 'find nil))
+	  ;;
+	  (set-charset-priority 'latin-iso8859-15)
+	  (setq ret (find-charset-region beg end))
+	  (if (equal ret '(ascii latin-iso8859-15))
+	      (throw 'find nil))
+	  ;;
+	  (set-charset-priority 'cyrillic-iso8859-5)
+	  (setq ret (find-charset-region beg end))
+	  (if (equal ret '(ascii cyrillic-iso8859-5))
+	      (throw 'find nil))
+	  ;;
+	  (set-charset-priority 'greek-iso8859-7)
+	  (setq ret (find-charset-region beg end))
+	  (if (equal ret '(ascii greek-iso8859-7))
+	      (throw 'find nil))
+	  ;;
+	  (apply 'set-charset-priority mew-charset-priority-list)
+	  (setq ret (find-charset-region beg end)))
+	(apply 'set-charset-priority charset-list)
+	ret))
+  (defalias 'mew-find-cs-region 'find-charset-region))
 
 ;; to internal
 (defun mew-cs-decode-region (beg end cs)
@@ -337,7 +387,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2008 Mew developing team.
+;; Copyright (C) 1997-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
