@@ -20,7 +20,7 @@
     ;; parameters used internally
     "server" "port" "ssh-server"
     "user" "auth-user" "auth-list"
-    "helo-domain" 
+    "helo-domain"
     "status" "process" "ssh-process" "ssl-process"
     "qfld" "messages"
     ;; parameters used internally and should be initialized
@@ -59,10 +59,10 @@
     ("content"       ("250" . "done"))
     ("quit"          (t     . "noop"))))
 
-(defsubst mew-smtp-fsm-by-status (status)
+(defun mew-smtp-fsm-by-status (status)
   (assoc status mew-smtp-fsm))
 
-(defsubst mew-smtp-fsm-next (status code)
+(defun mew-smtp-fsm-next (status code)
   (cdr (mew-assoc-match2 code (cdr (mew-smtp-fsm-by-status status)) 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -249,10 +249,10 @@
     ("PLAIN"    mew-smtp-command-auth-plain)     ;; RFC 2595
     ("LOGIN"    mew-smtp-command-auth-login)))   ;; No spec
 
-(defsubst mew-smtp-auth-get-func (auth)
+(defun mew-smtp-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-smtp-auth-alist 0)))
 
-;;
+;; CRAM MD5
 
 (defun mew-smtp-command-auth-cram-md5 (pro pnm)
   (mew-smtp-process-send-string pro "AUTH CRAM-MD5")
@@ -269,12 +269,7 @@
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
     (mew-smtp-process-send-string pro cram-md5)))
 
-;; There are two variations for SMTP AUTH LOGIN.
-;; 2) AUTH PLAIN then id then password
-;; 1) AUTH PLAIN id then password
-;;
-;; 1) is popular however.
-;; And a server of an ISP supports 1) only.
+;; LOGIN
 
 (defun mew-smtp-command-auth-login (pro pnm)
   (mew-smtp-process-send-string pro "AUTH LOGIN")
@@ -334,9 +329,9 @@
 
 (defconst mew-smtp-info-prefix "mew-smtp-info-")
 
-(defsubst mew-smtp-info-name (case &optional fallbacked)
+(defun mew-smtp-info-name (case &optional fallbacked)
   (let ((server (mew-smtp-server case))
-	(port (mew-smtp-port case))
+	(port (mew-*-to-string (mew-smtp-port case)))
 	(user (mew-smtp-user-only case))
 	(sshsrv (mew-smtp-ssh-server case))
 	(name mew-smtp-info-prefix))
@@ -348,7 +343,7 @@
 	       (fboundp 'make-network-process)
 	       (string= port "smtp"))
       (setq port "submission"))
-    (unless (string= port mew-smtp-port)
+    (unless (mew-port-equal port mew-smtp-port)
       (setq name (concat name ":" port)))
     (if sshsrv
 	(concat name "%" sshsrv)
@@ -404,17 +399,17 @@
   (defalias 'mew-open-network-stream 'open-network-stream))
 
 (defun mew-smtp-open (pnm server port)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     ;; xxx some OSes do not define "submission", sigh.
     (when (and (stringp sprt) (string= sprt "submission"))
-      (setq sprt (string-to-int (mew-serv-to-port sprt))))
+      (setq sprt (mew-serv-to-port sprt)))
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-smtp-timeout-time nil 'mew-smtp-timeout))
 	  (message "Connecting to the SMTP server...")
 	  (setq pro (mew-open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (message "Connecting to the SMTP server...done"))
       (quit
@@ -437,14 +432,14 @@
 (defun mew-smtp-send-message (case qfld msgs &optional fallbacked)
   (let ((server (mew-smtp-server case))
         (user (mew-smtp-user-only case))
-	(port (mew-smtp-port case))
+	(port (mew-*-to-string (mew-smtp-port case)))
 	(pnm (mew-smtp-info-name case fallbacked))
 	(sshsrv (mew-smtp-ssh-server case))
 	(sslp (mew-smtp-ssl case))
 	(sslport (mew-smtp-ssl-port case))
 	mew-inherit-submission
 	process sshname sshpro sslname sslpro lport tlsp tls fallback)
-    (when (and sslp (string= port sslport))
+    (when (and sslp (mew-port-equal port sslport))
       (setq tlsp t)
       ;; let stunnel know that a wrapper protocol is SMTP
       (setq tls mew-tls-smtp))
@@ -454,14 +449,14 @@
 	       (not fallbacked)
 	       mew-use-submission
 	       (fboundp 'make-network-process) ;; Emacs 22 or later
-	       (string= port "smtp"))
+	       (mew-port-equal port "smtp"))
       (setq port "submission")
       (setq fallback t)
       (unless tlsp
 	;; TLS uses stunnel. So, we should not use non-blocking connect.
 	;; Timeout should be carried out by stunnel.
 	(setq mew-inherit-submission t))
-      (if (string= sslport "smtp") (setq sslport "submission")))
+      (if (mew-port-equal sslport "smtp") (setq sslport "submission")))
     (cond
      (sshsrv
       (setq sshpro (mew-open-ssh-stream case server port sshsrv))
@@ -490,7 +485,7 @@
 		   (not fallbacked)
 		   mew-use-submission
 		   (fboundp 'make-network-process)
-		   (string= port "submission"))
+		   (mew-port-equal port "submission"))
 	      (progn
 		;; make-network-process with :nowait t sometime
 		;; returns nil, why?
@@ -530,7 +525,7 @@
 	(run-hooks 'mew-smtp-flush-hook)
 	(message "Flushing %s..." qfld)
 	(mew-smtp-send-message case qfld msgs)))))
-  
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Filter and sentinel
@@ -549,34 +544,34 @@
 	 (str (concat (mew-smtp-get-string pnm) string))
 	 (buf (process-buffer process))
 	 next func code)
-    (mew-smtp-debug (upcase status) string)
-    (if (and buf (get-buffer buf)) (set-buffer buf))
-    ;; SMTP server's strings should be short enough.
-    (mew-smtp-set-string pnm str)
-    (cond
-     ((and (string-match "\n$" str)
-	   (string-match "^\\([1-5][0-7][0-9]\\) " str))
-      (setq code (match-string 1 str))
-      (setq next (mew-smtp-fsm-next status code))
+    (save-excursion
+      (mew-smtp-debug (upcase status) string)
+      (if (and buf (get-buffer buf)) (set-buffer buf))
+      ;; SMTP server's strings should be short enough.
+      (mew-smtp-set-string pnm str)
       (cond
-       (next
-	(mew-smtp-set-status pnm next)
-	(setq func (intern-soft (concat "mew-smtp-command-" next)))
-	(and func (funcall func process pnm))
-	(mew-smtp-set-string pnm nil))
-       (t
-	(if (string-match "^pwd-" status)
-	    (mew-smtp-set-error pnm "SMTP password is wrong!")
-	  (if (string-match "\n$" str)
-	      (setq str (substring str 0 -1)))
-	  (unless (string-match "[.!]$" str)
-	    (setq str (concat str "."))))
-	(mew-smtp-set-error pnm str)
-	(mew-smtp-command-quit2 process pnm))))
-     (t ())))) ;; stay
+       ((and (string-match "\n$" str)
+	     (string-match "^\\([1-5][0-7][0-9]\\) " str))
+	(setq code (match-string 1 str))
+	(setq next (mew-smtp-fsm-next status code))
+	(cond
+	 (next
+	  (mew-smtp-set-status pnm next)
+	  (setq func (intern-soft (concat "mew-smtp-command-" next)))
+	  (and func (funcall func process pnm))
+	  (mew-smtp-set-string pnm nil))
+	 (t
+	  (if (string-match "^pwd-" status)
+	      (mew-smtp-set-error pnm "SMTP password is wrong!")
+	    (if (string-match "\n$" str)
+		(setq str (substring str 0 -1)))
+	    (unless (string-match "[.!]$" str)
+	      (setq str (concat str "."))))
+	  (mew-smtp-set-error pnm str)
+	  (mew-smtp-command-quit2 process pnm))))
+       (t ()))))) ;; stay
 
 (defun mew-smtp-sentinel (process event)
-  (mew-smtp-debug "SMTP SENTINEL" event)
   (let* ((pnm (process-name process))
 	 (pro (mew-smtp-get-process pnm))
 	 (case (mew-smtp-get-case pnm))
@@ -584,24 +579,26 @@
 	 (msgs (mew-smtp-get-messages pnm))
 	 (fallback (mew-smtp-get-fallback pnm))
 	 (status (mew-smtp-get-status pnm)))
-    (cond
-     ((or (string-match "failed" event)
-	  (and (string= status "greeting")
-	       (string-match "connection broken by remote peer" event)))
-      (if fallback
-	  (progn
-	    (mew-smtp-send-message case qfld msgs fallback)
-	    ;; A failed process stays
-	    (when (memq (process-status pro) '(failed connect))
-	      (delete-process pro)
-	      (mew-info-clean-up pnm)))
-	(mew-smtp-set-error pnm (substring event 0 -1))
-	(mew-smtp-sentinel2 process event)))
-     ((string-match "open" event)
-      ;; OK connected
-      )
-     (t
-      (mew-smtp-sentinel2 process event)))))
+    (save-excursion
+      (mew-smtp-debug "SMTP SENTINEL" event)
+      (cond
+       ((or (string-match "failed" event)
+	    (and (string= status "greeting")
+		 (string-match "connection broken by remote peer" event)))
+	(if fallback
+	    (progn
+	      (mew-smtp-send-message case qfld msgs fallback)
+	      ;; A failed process stays
+	      (when (memq (process-status pro) '(failed connect))
+		(delete-process pro)
+		(mew-info-clean-up pnm)))
+	  (mew-smtp-set-error pnm (substring event 0 -1))
+	  (mew-smtp-sentinel2 process event)))
+       ((string-match "open" event)
+	;; OK connected
+	)
+       (t
+	(mew-smtp-sentinel2 process event))))))
 
 (defun mew-smtp-sentinel2 (process event)
   (let* ((pnm (process-name process))
@@ -613,39 +610,40 @@
 	 (error (mew-smtp-get-error pnm))
 	 (sshpro (mew-smtp-get-ssh-process pnm))
 	 (sslpro (mew-smtp-get-ssl-process pnm)))
-    (cond
-     (error
-      (message "%s  This mail has been queued to %s" error qfld)
-      (when buf
-	;; A message file is not inserted at the beginning of the SMTP
-	;; session.
-	(set-buffer buf)
-	(mew-smtp-queue case error))
-      (mew-smtp-log pnm error))
-     (done
-      (message "Sending in background...done"))
-     (t
-      (if (null buf)
-	  (message "SMTP connection is lost")
-	(set-buffer buf)
-	(mew-smtp-queue case "SMTP connection is lost"))))
-    (mew-info-clean-up pnm)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (run-hooks 'mew-smtp-sentinel-hook)
-    (if (and done imapp) (mew-imap2-fcc case))
-    (when buf (mew-remove-buffer buf))))
+    (save-excursion
+      (cond
+       (error
+	(message "%s  This mail has been queued to %s" error qfld)
+	(when buf
+	  ;; A message file is not inserted at the beginning of the SMTP
+	  ;; session.
+	  (set-buffer buf)
+	  (mew-smtp-queue case error pnm))
+	(mew-smtp-log pnm error))
+       (done
+	(message "Sending in background...done"))
+       (t
+	(if (null buf)
+	    (message "SMTP connection is lost")
+	  (set-buffer buf)
+	  (mew-smtp-queue case "SMTP connection is lost" pnm))))
+      (mew-info-clean-up pnm)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (run-hooks 'mew-smtp-sentinel-hook)
+      (if (and done imapp) (mew-imap2-fcc case))
+      (when buf (mew-remove-buffer buf)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Queuing
 ;;;
 
-(defun mew-smtp-queue (case err)
+(defun mew-smtp-queue (case err &optional apnm)
   ;; Must be in a buffer where a message is contained.
-  (let* ((pnm (mew-smtp-info-name case))
+  (let* ((pnm (or apnm (mew-smtp-info-name case)))
 	 (qfld (mew-queue-folder case))
 	 (oname (buffer-name))
 	 (work (buffer-file-name))
@@ -660,11 +658,9 @@
     ;;
     (mew-smtp-set-recipients pnm (mew-smtp-get-orig-recipients pnm))
     (let* ((n mew-smtp-info-list-save-length)
-	   (data (make-vector n nil))
-	   (i 0))
-      (while (< i n)
-	(aset data i (aref (mew-info pnm) i))
-	(setq i (1+ i)))
+	   (data (make-vector n nil)))
+      (dotimes (i n)
+	(aset data i (aref (mew-info pnm) i)))
       (mew-lisp-save info data))
     ;;
     (message "%s has been queued to %s (%s)" oname nname err)
@@ -679,7 +675,8 @@
 (defun mew-smtp-bcc (pro pnm back)
   (let* ((dir (file-name-directory back))
 	 (msg (file-name-nondirectory back))
-	 (case (mew-smtp-get-case pnm)))
+	 (case (mew-smtp-get-case pnm))
+	 msgid logtime)
     (mew-elet
      (mew-erase-buffer)
      (mew-set-buffer-multibyte t)
@@ -687,7 +684,9 @@
      (mew-smtp-set-orig-recipients pnm (mew-smtp-get-bcc pnm))
      (mew-smtp-set-bcc pnm nil)
      ;;
-     (mew-encode-id-date pnm (mew-smtp-message-id case)) ;; save-excursion
+     (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-smtp-message-id case))) ;; save-excursion
+     (mew-smtp-set-msgid pnm msgid)
+     (mew-smtp-set-logtime pnm logtime)
      (goto-char (point-max))
      (mew-draft-header-insert mew-to: "Bcc-Receiver:;")
      (mew-draft-header-insert mew-subj: mew-bcc-subject)
@@ -695,7 +694,8 @@
      (mew-draft-header-insert mew-organization: (mew-organization case))
      (mew-draft-header-insert-alist (mew-header-alist case))
      ;; X-Mailer: must be the last
-     (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+     (if (mew-use-x-mailer case)
+	 (mew-draft-header-insert mew-x-mailer: mew-x-mailer))
      (mew-header-set "\n")
      (insert mew-bcc-body)
      (goto-char (mew-header-end))
@@ -726,12 +726,13 @@
 	(msgid (mew-smtp-get-msgid pnm))
 	(recipients (mew-smtp-get-orig-recipients pnm))
 	(server (mew-smtp-get-server pnm))
+	(port (mew-smtp-get-port pnm))
 	(sshsrv (mew-smtp-get-ssh-server pnm))
 	(sslp (mew-smtp-get-ssl-process pnm)))
     (with-temp-buffer
       (and logtime (insert logtime))
       (and msgid (insert " id=" msgid))
-      (and server (insert " server=" server))
+      (and server (insert " server=" server ":" port))
       (and sshsrv (insert " sshsrv=" sshsrv))
       (and sslp (insert " SSL/TLS"))
       (and recipients
@@ -751,13 +752,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2006 Mew developing team.
+;; Copyright (C) 1999-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -766,7 +767,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
