@@ -19,9 +19,6 @@
       (mew-elet
        (goto-char (point-max))
        (unless (bolp) (insert "\n"))
-       (if (and mew-xemacs-p (extent-at (point) nil nil nil 'at))
-	   ;; to skip graphics
-	   (insert "\n"))
        (mew-message-clear-end-of)
        (when (or mew-end-of-message-string mew-end-of-part-string)
 	 (move-overlay (mew-minfo-get-eom) (point-max) (point-max))
@@ -31,7 +28,7 @@
 	       (mew-message-set-end-of-part))
 	   (mew-message-set-end-of-message)))))))
 
-(defsubst mew-message-clear-end-of ()
+(defun mew-message-clear-end-of ()
   (unless (overlayp (mew-minfo-get-eom))
     (mew-minfo-set-eom (mew-overlay-make (point-max) (point-max))))
   (mew-message-set-end-of-nil))
@@ -98,8 +95,8 @@ If nodisplay is non-nil, displays the cached message."
 ;;; After step
 ;;;
 
-(defsubst mew-summary-display-after (direction)
-  (cond 
+(defun mew-summary-display-after (direction)
+  (cond
    ((eq direction 'down)
     (mew-summary-display-down))
    ((eq direction 'up)
@@ -119,19 +116,21 @@ If nodisplay is non-nil, displays the cached message."
   (if (or mew-summary-recenter-p (interactive-p))
       (recenter (/ (- (window-height) 2) 2))))
 
-(defsubst mew-summary-display-preamble ()
+(defun mew-summary-display-preamble ()
   ;; message buffer
   (mew-erase-buffer)
   (mew-message-clear-end-of)
   (mew-overlay-delete-buffer) ;; veil, xxx also delete extents?
   ;; kill mark for cite
-  (and (mark-marker) (set-marker (mark-marker) nil)))
+  (and (mark-marker) (set-marker (mark-marker) nil))
+  (mew-normal-line))
 
 (defvar mew-message-last-buffer nil)
 
-(defsubst mew-summary-display-postscript (&optional no-hook)
+(defun mew-summary-display-postscript (&optional no-hook)
   ;; message buffer
   (unless no-hook (run-hooks 'mew-message-hook))
+  (mew-minfo-set-reob nil)
   (setq mew-message-last-buffer (current-buffer))
   (mew-message-set-end-of)
   (set-buffer-modified-p nil))
@@ -143,6 +142,7 @@ If nodisplay is non-nil, displays the cached message."
   (mew-highlight-cursor-line)
   (when (and mew-use-unread-mark
 	     (mew-summary-message-number)
+	     ;; mark would be nil. MUST be equal
 	     (equal (mew-summary-get-mark) mew-mark-unread))
     (unless (and mew-inherit-after-marking
 		 (not mew-delete-unread-mark-by-mark))
@@ -154,33 +154,34 @@ If nodisplay is non-nil, displays the cached message."
 ;;; Displaying
 ;;;
 
-(defsubst mew-summary-message-toobig (fld msg)
-  (let ((file (mew-expand-folder fld msg)))
+(defun mew-summary-message-toobig (fld msg)
+  (let ((file (mew-expand-msg fld msg)))
     (and (file-readable-p file)
 	 (> (mew-file-get-size file) mew-file-max-size))))
 
 (defun mew-summary-display (&optional redisplay)
-  " (1) If called interactively, this command lets you read through
-messages.  That is, display a message, scroll it, and
+  " (1) If called interactively, this command lets you read
+through messages.  That is, display a message, scroll it, and
 move-then-display another message or part.
 
-See 'mew-summary-show-direction' to set the direction that the cursor
-moves. You can select a value out of 'up, 'down, 'next(current
-direction) or 'stop.  Default is 'down. 'mew-summary-show-direction'
-is valid in this case only because the cursor stays in the other
-cases.
+See 'mew-summary-show-direction' to set the direction that the
+cursor moves. You can select a value out of 'up, 'down,
+'next(current direction) or 'stop.  Default is
+'down. 'mew-summary-show-direction' is valid in this case only
+because the cursor stays in the other cases.
 
  (2) If called interactively with '\\[universal-argument]' (i.e.
-REDISPLAY is 't'), this command displays the current message or part
-again. This is a convenient way to get back to the beginning of the
-current message or part.
+REDISPLAY is non-nil), this command displays the current message
+or part again. This is a convenient way to get back to the
+beginning of the current message or part.
 
- (3) If called internally, this function displays the current message
-or part. If it is already displayed, nothing changes.
+ (3) If called internally, this function displays the current
+message or part. If it is already displayed, nothing changes.
 
- (4) If called internally and REDISPLAY is 't', this function displays
-the current message or part. Even if it is already displayed, this
-function displays it again getting back to the beginning."
+ (4) If called internally and REDISPLAY is 'non-nil', this
+function displays the current message or part. Even if it is
+already displayed, this function displays it again getting back
+to the beginning."
   (interactive "P")
   (when (or redisplay (mew-sinfo-get-disp-msg) (interactive-p))
     (mew-summary-msg-or-part
@@ -188,9 +189,10 @@ function displays it again getting back to the beginning."
 	    (vfld (mew-summary-folder-name 'ext))
 	    (msg (mew-summary-message-number))
 	    (part (mew-syntax-nums))
-	    (ofld (mew-current-get-fld (mew-frame-id)))
-	    (omsg (mew-current-get-msg (mew-frame-id)))
-	    (opart (mew-current-get-part (mew-frame-id)))
+	    (fid (mew-frame-id))
+	    (ofld (mew-current-get-fld fid))
+	    (omsg (mew-current-get-msg fid))
+	    (opart (mew-current-get-part fid))
 	    (cache (mew-cache-hit fld (or msg omsg)))
 	    (win (selected-window))
 	    (read-through (interactive-p))
@@ -345,7 +347,7 @@ This function does not create a cache."
 ;;;
 
 (defun mew-summary-analyze-again-alternative (&optional arg)
-  "This command analyzes the message again with 
+  "This command analyzes the message again with
 'mew-use-alternative' and 'mew-use-text-body' reversed."
   (interactive "P")
   (mew-summary-goto-message)
@@ -430,7 +432,7 @@ displays it."
   "Insert this message or part into Message mode in the raw format.
 This command treats PGP/MIME transparent. That is an encrypted/signed
 part is displayed as a decrypted/verified part. Also, if this command
-is used for a message whose entire body is encrypted/signed, 
+is used for a message whose entire body is encrypted/signed,
 a decrypted/verified body is displayed."
   (interactive)
   (mew-summary-msg-or-part
@@ -462,7 +464,7 @@ a decrypted/verified body is displayed."
 	   ;; message buffer
 	   (mew-current-set fld msg nums)
 	   (mew-minfo-set-summary vfld)
-	   ;; The following codes are not necessary. 
+	   ;; The following codes are not necessary.
 	   ;; (mew-decode-syntax-clear)
 	   ;; (mew-decode-syntax-delete)
 	   (mew-elet
@@ -499,14 +501,13 @@ If called with '\\[universal-argument]', it stays writable."
   (mew-summary-msg-or-part
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
-	  (file (mew-expand-folder fld msg)))
+	  (file (mew-expand-msg fld msg)))
      (when (mew-sinfo-get-disp-msg)
        (mew-summary-toggle-disp-msg))
-     (mew-frwlet
-      (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
-      (if arg
-	  (switch-to-buffer (mew-find-file-noselect2 file))
-	(view-buffer (mew-find-file-noselect2 file) 'kill-buffer))))))
+     (mew-frwlet (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
+       (if arg
+	   (switch-to-buffer (mew-find-file-noselect2 file))
+	 (view-buffer (mew-find-file-noselect2 file) 'kill-buffer))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -520,67 +521,61 @@ If called with '\\[universal-argument]', it stays writable."
    (let* ((win (selected-window))
 	  (fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
-	  lst ent svr-date snd rcv svr date tmp)
-     (let ((mew-use-full-window nil))
-       (mew-summary-display nil))
-     (save-excursion
-       (set-buffer (or (save-excursion
-			 (set-buffer (mew-buffer-message))
-			 (if (mew-msghder-p) (current-buffer)))
-		       ;; header exists only in cache if multipart
-		       (mew-cache-hit fld msg)))
-       (setq lst (mew-header-get-value mew-received: 'as-list)))
-     (mew-summary-toggle-disp-msg 'on)
-     (mew-window-configure 'message)
+	  lst svr-date (snd "?") (rcv "?") last svr date tmp)
+     (mew-summary-set-message-buffer fld msg)
+     ;; message buffer or cache buffer
+     (setq lst (nreverse (mew-header-get-value mew-received: 'as-list)))
+     (set-buffer (mew-buffer-message))
      ;; message buffer
      (mew-summary-display-preamble)
-     (setq lst (nreverse lst))
-     (while lst
-       (setq ent (car lst))
-       (setq lst (cdr lst))
-       (setq svr-date (mew-split ent ?\;))
-       (mew-set '(svr date) svr-date)
-       (unless date
-	 ;; for broken MTAs
-	 (if (string-match "Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun" ent)
-	     (setq date (substring ent (match-beginning 0)))))
-       (if date (setq date (mew-time-rfc-to-sortkey date 'tzadj)))
-       (unless date (setq date "19700101000000"))
-       (setq tmp (cons (list date svr) tmp))
-       (cond
-	((string-match "(\\(qmail [0-9]+\\)" svr)
-	 (setq rcv (match-string 1 svr)))
-	((string-match "by[ \t]*\\([^() \t\n]+\\)" svr)
-	 (setq rcv (match-string 1 svr))
-	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
-	     (setq snd (match-string 1 svr)))
-	 (when (and (string= snd "unknown")
-		    (string-match "from unknown (HELO \\([^() \t\n]+\\))" svr))
-	   (setq snd (concat (match-string 1 svr) "(unknown)"))))
-	((string-match "[ \t]*\\([^() \t\n]+\\)" svr)
-	 (setq rcv (match-string 1 svr))
-	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
-	     (setq snd (match-string 1 svr)))))
-       (mew-elet
-	(insert date)
-	(beginning-of-line)
-	(forward-char 4)
-	(insert "/")
-	(forward-char 2)
-	(insert "/")
-	(forward-char 2)
-	(insert " ")
-	(forward-char 2)
-	(insert ":")
-	(forward-char 2)
-	(insert ":")
-	(forward-char 2)
-	(insert " (" (mew-time-tmzn-int) ") ")
-	(if snd (insert snd))
-	(insert " => ")
-	(if rcv (insert rcv))
-	(insert "\n")
-	(setq snd nil rcv nil)))
+     (save-excursion
+       (dolist (ent lst)
+	 (setq svr-date (mew-split ent ?\;))
+	 (mew-set '(svr date) svr-date)
+	 (unless date
+	   ;; for broken MTAs
+	   (if (string-match "Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun" ent)
+	       (setq date (substring ent (match-beginning 0)))))
+	 (if date (setq date (mew-time-rfc-to-sortkey date 'tzadj)))
+	 (unless date (setq date "19700101000000"))
+	 (setq tmp (cons (list date svr) tmp))
+	 (cond
+	  ((string-match "(\\(qmail [0-9]+\\)" svr)
+	   (setq rcv (match-string 1 svr)))
+	  ((string-match "by[ \t]+\\([^() \t\n]+\\)" svr)
+	   (setq rcv (match-string 1 svr))
+	   (if (string-match "from[ \t]+\\([^() \t\n]+\\).*\\([[][.0-9]+[]]\\)" svr)
+	       (setq snd (format "%s %s" (match-string 1 svr) (match-string 2 svr)))
+	     (if (string-match "from[ \t]+\\([^() \t\n]+\\)" svr)
+		 (setq snd (match-string 1 svr))))
+	   (when (and (string= snd "unknown")
+		      (string-match "from unknown (HELO \\([^() \t\n]+\\))" svr))
+	     (setq snd (concat (match-string 1 svr) "(unknown)"))))
+	  ((string-match "[ \t]*\\([^() \t\n]+\\)" svr)
+	   (setq rcv (match-string 1 svr))
+	   (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
+	       (setq snd (match-string 1 svr)))))
+	 (mew-elet
+	  (insert date)
+	  (beginning-of-line)
+	  (forward-char 4)
+	  (insert "/")
+	  (forward-char 2)
+	  (insert "/")
+	  (forward-char 2)
+	  (insert " ")
+	  (forward-char 2)
+	  (insert ":")
+	  (forward-char 2)
+	  (insert ":")
+	  (forward-char 2)
+	  (insert " (" (mew-time-tmzn-int) ") ")
+	  (if (and snd (not (string= snd last))) (insert snd))
+	  (insert " => ")
+	  (if rcv (insert rcv))
+	  (insert "\n")
+	  (setq last rcv)
+	  (setq snd "?" rcv "?"))))
      (mew-summary-display-postscript 'no-hook)
      (select-window win))))
 
@@ -612,16 +607,14 @@ ad-hoc solution for putting the review mark on a thread.")
 
 (defun mew-summary-down ()
   (funcall mew-summary-down-function)
-  (cond 
+  (cond
    ((mew-re-search-forward-visibile mew-regex-msg-or-part)
     (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'down)
     t)
    (t
     (mew-decode-syntax-delete)
-    (forward-line -1)
-    (while (and (eq (get-text-property (point) 'invisible) t)
-		(= (forward-line -1) 0)))
+    (forward-line -1) ;; for invisible
     (unless (pos-visible-in-window-p)
       ;; This ensures that some messages are displayed in Summary.
       (recenter))
@@ -632,12 +625,12 @@ ad-hoc solution for putting the review mark on a thread.")
 
 (defun mew-summary-up ()
   (beginning-of-line)
-  (cond 
+  (cond
    ((mew-re-search-backward-visible mew-regex-msg-or-part)
     (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'up)
     t)
-   (t 
+   (t
     (mew-thread-move-cursor)
     (mew-decode-syntax-delete)
     (mew-summary-reset)
@@ -650,10 +643,10 @@ ad-hoc solution for putting the review mark on a thread.")
 ;;;
 
 (defun mew-summary-display-next ()
-  (if (mew-summary-next) (mew-summary-display nil)))
+  (if (mew-summary-next) (mew-summary-display)))
 
 (defun mew-summary-display-up (&optional arg)
-  "Move to above then display. Targets includes parts, messages 
+  "Move to above then display. Targets includes parts, messages
 marked with '*', and non-marked messages. When called with '\\[universal-argument]',
 parts are skipped."
   (interactive "P")
@@ -661,17 +654,17 @@ parts are skipped."
   (when arg
     (mew-summary-goto-message)
     (mew-decode-syntax-delete))
-  (if (mew-summary-up) (mew-summary-display nil)))
+  (if (mew-summary-up) (mew-summary-display)))
 
 (defun mew-summary-display-down (&optional arg)
-  "Move to below then display. Targets includes parts, messages 
+  "Move to below then display. Targets includes parts, messages
 marked with '*', and non-marked messages. When called with '\\[universal-argument]',
 parts are skipped."
   (interactive "P")
   (when arg
     (mew-summary-goto-message)
     (mew-decode-syntax-delete))
-  (if (mew-summary-down) (mew-summary-display nil)))
+  (if (mew-summary-down) (mew-summary-display)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -693,11 +686,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   "Make this message scroll up with one line."
   (interactive)
   (mew-summary-msg-or-part
-   (let ((win (selected-window))
-	 (msg (mew-summary-message-number))
-	 (part (mew-syntax-nums))
-	 (omsg (mew-current-get-msg (mew-frame-id)))
-	 (opart (mew-current-get-part (mew-frame-id))))
+   (let* ((win (selected-window))
+	  (msg (mew-summary-message-number))
+	  (part (mew-syntax-nums))
+	  (fid (mew-frame-id))
+	  (omsg (mew-current-get-msg fid))
+	  (opart (mew-current-get-part fid)))
      ;; xxx how about folder check?
      (if (or (and msg (string= msg omsg) (null part) (null opart))
 	     (and part (equal part opart))) ;; MUST be equal
@@ -714,11 +708,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   "Make this message scroll down with one line."
   (interactive)
   (mew-summary-msg-or-part
-   (let ((win (selected-window))
-	 (msg (mew-summary-message-number))
-	 (part (mew-syntax-nums))
-	 (omsg (mew-current-get-msg (mew-frame-id)))
-	 (opart (mew-current-get-part (mew-frame-id))))
+   (let* ((win (selected-window))
+	  (msg (mew-summary-message-number))
+	  (part (mew-syntax-nums))
+	  (fid (mew-frame-id))
+	  (omsg (mew-current-get-msg fid))
+	  (opart (mew-current-get-part fid)))
      ;; xxx how about folder check?
      (if (or (and msg (string= msg omsg) (null part) (null opart))
 	     (and part (equal part opart))) ;; MUST be equal
@@ -742,7 +737,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-previous-line (- arg))
-    (let ((col (current-column)))
+    (let (col)
+      (if (eq last-command this-command)
+	  (setq col (mew-sinfo-get-column)))
+      (unless col
+	(setq col (current-column))
+	(mew-sinfo-set-column col))
       (while (> arg 0)
 	(forward-line)
 	(while (or (looking-at mew-regex-thread-separator)
@@ -757,9 +757,14 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-next-line (- arg))
-    (let ((col (current-column)))
+    (let (col)
+      (if (eq last-command this-command)
+	  (setq col (mew-sinfo-get-column)))
+      (unless col
+	(setq col (current-column))
+	(mew-sinfo-set-column col))
       (while (> arg 0)
-	(forward-line -1)
+	(forward-line -1) ;; for invisible
 	(while (or (looking-at mew-regex-thread-separator)
 		   (eq (get-text-property (point) 'invisible) t))
 	  (forward-line -1))
@@ -782,13 +787,13 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -797,7 +802,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
