@@ -632,7 +632,7 @@ message."
 	       (aset vec idx num)
 	     (error "invalid number")))
 	 (unless TTL (error "no total"))
-	 (dotimes (< i len)
+	 (dotimes (i len)
 	   (unless (stringp (aref vec i)) (error "Not enough fragments")))
 	 ;; now reassemble
 	 (mew-erase-buffer)
@@ -1168,7 +1168,7 @@ Executing this command enables searching such information."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
