@@ -29,98 +29,98 @@
 ;;
 ;;
 
-(defsubst mew-syntax-singlepart-p (syntax)
+(defun mew-syntax-singlepart-p (syntax)
   (eq (aref syntax 0) 'single))
 
-(defsubst mew-syntax-multipart-p (syntax)
+(defun mew-syntax-multipart-p (syntax)
   (eq (aref syntax 0) 'multi))
 
-(defsubst mew-syntax-message-p (syntax)
+(defun mew-syntax-message-p (syntax)
   (eq (aref syntax 0) 'message))
 
 ;;
 ;;
 ;;
 
-(defsubst mew-syntax-get-key (syntax)
+(defun mew-syntax-get-key (syntax)
   (aref syntax 0))
 
-(defsubst mew-syntax-get-begin (syntax)
+(defun mew-syntax-get-begin (syntax)
   (let ((beg (aref syntax 1)))
     (if (listp beg) (car beg) beg)))
 
-(defsubst mew-syntax-get-begin2 (syntax)
+(defun mew-syntax-get-begin2 (syntax)
   (let ((beg (aref syntax 1)))
     (if (listp beg) (cdr beg) beg)))
 
-(defsubst mew-syntax-get-end (syntax)
+(defun mew-syntax-get-end (syntax)
   (aref syntax 2))
 
-(defsubst mew-syntax-get-privacy (syntax)
+(defun mew-syntax-get-privacy (syntax)
   (aref syntax 3))
 
-(defsubst mew-syntax-get-ct (syntax)
+(defun mew-syntax-get-ct (syntax)
   (aref syntax 4))
 
-(defsubst mew-syntax-get-cte (syntax)
+(defun mew-syntax-get-cte (syntax)
   (aref syntax 5))
 
-(defsubst mew-syntax-get-cd (syntax)
+(defun mew-syntax-get-cd (syntax)
   (aref syntax 6))
 
-(defsubst mew-syntax-get-cid (syntax)
+(defun mew-syntax-get-cid (syntax)
   (aref syntax 7))
 
-(defsubst mew-syntax-get-cdp (syntax)
+(defun mew-syntax-get-cdp (syntax)
   (aref syntax 8))
 
-(defsubst mew-syntax-get-part (syntax)
+(defun mew-syntax-get-part (syntax)
   (aref syntax 9))
 
 
-(defsubst mew-syntax-set-key (syntax key)
+(defun mew-syntax-set-key (syntax key)
   (aset syntax 0 key))
 
-(defsubst mew-syntax-set-begin (syntax begin)
+(defun mew-syntax-set-begin (syntax begin)
   (aset syntax 1 begin))
 
-(defsubst mew-syntax-set-end (syntax end)
+(defun mew-syntax-set-end (syntax end)
   (aset syntax 2 end))
 
-(defsubst mew-syntax-set-privacy (syntax privacy)
+(defun mew-syntax-set-privacy (syntax privacy)
   (aset syntax 3 privacy))
 
-(defsubst mew-syntax-set-ct (syntax ct)
+(defun mew-syntax-set-ct (syntax ct)
   (aset syntax 4 ct))
 
-(defsubst mew-syntax-set-cte (syntax cte)
+(defun mew-syntax-set-cte (syntax cte)
   (aset syntax 5 cte))
 
-(defsubst mew-syntax-set-cd (syntax cd)
+(defun mew-syntax-set-cd (syntax cd)
   (aset syntax 6 cd))
 
-(defsubst mew-syntax-set-cid (syntax cid)
+(defun mew-syntax-set-cid (syntax cid)
   (aset syntax 7 cid))
 
-(defsubst mew-syntax-set-cdp (syntax cdp)
+(defun mew-syntax-set-cdp (syntax cdp)
   (aset syntax 8 cdp))
 
-(defsubst mew-syntax-set-part (syntax part)
+(defun mew-syntax-set-part (syntax part)
   (aset syntax 9 part))
 
 
 ;; alias for encode syntax
 
-(defsubst mew-syntax-get-file (syntax)
+(defun mew-syntax-get-file (syntax)
   (aref syntax 1))
 
-(defsubst mew-syntax-set-file (syntax file)
+(defun mew-syntax-set-file (syntax file)
   (aset syntax 1 file))
 
-(defsubst mew-syntax-get-decrypters (syntax)
+(defun mew-syntax-get-decrypters (syntax)
   (aref syntax 2))
 
-(defsubst mew-syntax-set-decrypters (syntax decrypters)
+(defun mew-syntax-set-decrypters (syntax decrypters)
   (aset syntax 2 decrypters))
 
 ;; for content parameters
@@ -130,12 +130,12 @@
       (mew-capitalize (car ctl))
     (car ctl)))
 
-(defsubst mew-syntax-get-params (ctl)
+(defun mew-syntax-get-params (ctl)
   (cdr ctl))
 
 ;; ctl = (value (pname pvalue) (pname pvalue) ...)
 ;; ctl = ((pname pvalue) (pname pvalue) ...)
-(defsubst mew-syntax-get-param (ctl member)
+(defun mew-syntax-get-param (ctl member)
   (nth 1 (assoc member ctl)))
 
 (defun mew-syntax-treat-filename-default-function (file)
@@ -153,7 +153,7 @@
     file))
 
 ;; need to setq
-(defsubst mew-syntax-cat (syntax part)
+(defun mew-syntax-cat (syntax part)
   (vconcat syntax (vector part)))
 
 (defun mew-syntax-cdp-format (ct file &optional cdp force-file)
@@ -170,12 +170,11 @@
 ;; Encryption
 
 (defun mew-syntax-encrypted-p (syntax)
-  (let ((plist (mew-syntax-get-privacy (mew-syntax-get-part syntax))))
-    (catch 'loop
-      (while plist
-	(if (mew-case-equal (nth 0 (car plist)) mew-ct-mle)
-	    (throw 'loop t))
-	(setq plist (cdr plist))))))
+  (catch 'loop
+    (dolist (ent (mew-syntax-get-privacy (mew-syntax-get-part syntax)))
+      (if (or (mew-case-equal (mew-privacy-dinfo-get-ct ent) mew-ct-mle)
+	      (mew-case-equal (mew-privacy-dinfo-get-proto ent) mew-ct-smm-enc))
+	  (throw 'loop t)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -288,7 +287,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
-;; 
+;;
 ;;
 
 (defun mew-summary-goto-part (msg part)
@@ -309,37 +308,24 @@
   (beginning-of-line))
 
 (defun mew-syntax-number ()
-  (let ((event last-command-event)
-	ret str)
-    (when (and mew-xemacs-p
-	       mew-icon-p
-	       (mouse-event-p event)
-	       (event-over-toolbar-p event)
-	       (or (button-press-event-p event)
-		   (button-release-event-p event)))
-      (setq str (toolbar-button-help-string (event-toolbar-button event)))
-      ;; last-pressed-toolbar-button cannot be used.
-      (if (string-match mew-regex-part-icon str)
-	  (setq ret (mew-match-string 1 str))))
-    (or ret
-	(if (or (mew-in-attach-p)
-		(mew-in-decode-syntax-p))
-	    (save-excursion
-	      (beginning-of-line)
-	      (if (looking-at mew-regex-part) (mew-sumsyn-part)))))))
+  (if (or (mew-in-attach-p)
+	  (mew-in-decode-syntax-p))
+      (save-excursion
+	(beginning-of-line)
+	(if (looking-at mew-regex-part) (mew-sumsyn-part)))))
 
-(defsubst mew-syntax-number-to-nums (strnum)
+(defun mew-syntax-number-to-nums (strnum)
   (if strnum
-      (mapcar 'string-to-int (mew-split strnum ?.))
+      (mapcar 'string-to-number (mew-split strnum ?.))
     nil))
 
-(defsubst mew-syntax-nums ()
+(defun mew-syntax-nums ()
   (mew-syntax-number-to-nums (mew-syntax-number)))
 
-(defsubst mew-summary-goto-mark ()
+(defun mew-summary-goto-mark ()
   (beginning-of-line))
 
-(defsubst mew-summary-goto-body (&optional after)
+(defun mew-summary-goto-body (&optional after)
   (when mew-summary-form-body-starter
     (beginning-of-line)
     (let ((beg (point)))
@@ -358,9 +344,16 @@
 ;;; Summary mode syntax
 ;;;
 
+(defvar mew-use-cut-file-name t)
+
 (defun mew-summary-info (arg)
-  "Display the folder name of this message and the message number.
-If this is a cache message, both the unique id and the size are also shown."
+  "Display the folder name of this message, its number, its size
+and its unique id. If this is a cache message, both the unique
+id and the size are also shown.  If called with
+'\\[universal-argument]', its Message-ID: and its parent's
+Message-ID: are displayed instead.  If 'mew-use-cut-file-name' is
+non-nil, the file path is pushed to the cut buffer of your window
+system."
   (interactive "P")
   (mew-summary-msg
    (when (mew-sumsyn-match mew-regex-sumsyn-long)
@@ -368,13 +361,20 @@ If this is a cache message, both the unique id and the size are also shown."
 	 (message "my-id=%s par-id=%s"
 		  (mew-sumsyn-my-id)
 		  (mew-sumsyn-parent-id))
-       (message "%s/%s size=%s uid=%s"
-		(mew-sumsyn-folder-name)
-		(mew-sumsyn-message-number)
-		(mew-sumsyn-message-size)
-		(mew-sumsyn-message-uid))))))
+       (let* ((fld (mew-sumsyn-folder-name))
+	      (msg (mew-sumsyn-message-number))
+	      (size (mew-sumsyn-message-size))
+	      (uid (mew-sumsyn-message-uid))
+	      (file (concat (file-name-as-directory fld) msg))
+	      (path (mew-expand-folder file))
+	      (rpath (mew-msg-get-filename path)))
+	 (if (mew-virtual-p)
+	     (setq file (mew-folder-path-to-folder file 'has-proto)))
+	 (message "%s size=%s uid=%s" file size uid)
+	 (when mew-use-cut-file-name
+	   (kill-new rpath)))))))
 
-(defsubst mew-sumsyn-match (regex)
+(defun mew-sumsyn-match (regex)
   (save-excursion
     (beginning-of-line)
     (let ((start (point)))
@@ -382,24 +382,24 @@ If this is a cache message, both the unique id and the size are also shown."
       (when (search-backward "\r" start t)
 	(looking-at regex)))))
 
-(defsubst mew-sumsyn-folder-name ()
+(defun mew-sumsyn-folder-name ()
   (let ((fn (mew-match-string 1))
 	(bn (buffer-name)))
     (cond
      ((not (string= fn "")) fn)
-     ((mew-thread-p) ;; xxx
-      (mew-physical-folder bn))
+     ((mew-virtual-for-one-summary)
+      (mew-vinfo-get-physical-folder))
      (t bn))))
 
-(defsubst mew-sumsyn-message-number () (mew-match-string 2))
-(defsubst mew-sumsyn-my-id          () (mew-match-string 3))
-(defsubst mew-sumsyn-parent-id      () (mew-match-string 4))
-(defsubst mew-sumsyn-message-uid    () (mew-match-string 5))
-(defsubst mew-sumsyn-message-size   () (mew-match-string 6))
+(defun mew-sumsyn-message-number () (mew-match-string 2))
+(defun mew-sumsyn-my-id          () (mew-match-string 3))
+(defun mew-sumsyn-parent-id      () (mew-match-string 4))
+(defun mew-sumsyn-message-uid    () (mew-match-string 5))
+(defun mew-sumsyn-message-size   () (mew-match-string 6))
 
-(defsubst mew-sumsyn-part () (mew-match-string 1))
+(defun mew-sumsyn-part () (mew-match-string 1))
 
-(defsubst mew-summary-folder-name (&optional ext)
+(defun mew-summary-folder-name (&optional ext)
   (cond
    ((or ext (mew-summary-p))
     (buffer-name))
@@ -409,64 +409,45 @@ If this is a cache message, both the unique id and the size are also shown."
       (when (mew-sumsyn-match mew-regex-sumsyn-short)
 	(mew-sumsyn-folder-name))))))
 
-(defsubst mew-summary-message-number ()
-  (let ((event last-command-event)
-	ret str)
-    (if (and mew-xemacs-p
-	     mew-icon-p	
-	     ;; exclude press button 2 in summary buffer.
-	     ;; exclude pulldown menu in Summary mode.
-	     ;; exclude popup menu of multipart icon because
-	     ;; the cursor has already moved.
-	     (mouse-event-p event)
-	     (event-over-toolbar-p event)
-	     (or (button-press-event-p event)     ;; right button
-		 (button-release-event-p event))) ;; left button
-	(when last-pressed-toolbar-button
-	  (setq str (toolbar-button-help-string 
-		     last-pressed-toolbar-button))
-	  (if (string-match mew-regex-msg-icon str)
-	      (setq ret (mew-match-string 1 str)))))
-    (if ret
-	ret
-      (unless (mew-in-decode-syntax-p)
-	(when (mew-sumsyn-match mew-regex-sumsyn-short)
-	  (mew-sumsyn-message-number))))))
+(defun mew-summary-message-number ()
+  (unless (mew-in-decode-syntax-p)
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (mew-sumsyn-message-number))))
 
-(defsubst mew-summary-message-number2 ()
+(defun mew-summary-message-number2 ()
   (save-excursion
     (mew-summary-goto-message)
     (when (mew-sumsyn-match mew-regex-sumsyn-short)
       (mew-sumsyn-message-number))))
 
-(defsubst mew-summary-my-id ()
+(defun mew-summary-my-id ()
   (when (mew-sumsyn-match mew-regex-sumsyn-long)
     (mew-sumsyn-my-id)))
 
-(defsubst mew-summary-parent-id ()
+(defun mew-summary-parent-id ()
   (when (mew-sumsyn-match mew-regex-sumsyn-long)
     (mew-sumsyn-parent-id)))
 
-(defsubst mew-summary-message-uid ()
+(defun mew-summary-message-uid ()
   (when (mew-sumsyn-match mew-regex-sumsyn-long)
     (mew-sumsyn-message-uid)))
 
-(defsubst mew-summary-message-size ()
+(defun mew-summary-message-size ()
   (when (mew-sumsyn-match mew-regex-sumsyn-long)
     (mew-sumsyn-message-size)))
 
-(defsubst mew-syntax-change-message-number (msg)
+(defun mew-syntax-change-message-number (msg)
   (save-excursion
     (when (mew-sumsyn-match mew-regex-sumsyn-short)
       (mew-syntax-change-message-number2 msg))))
 
 ;; see also mew-sumsyn-message-number
-(defsubst mew-syntax-change-message-number2 (msg)
+(defun mew-syntax-change-message-number2 (msg)
   (delete-region (match-beginning 2) (match-end 2))
   (goto-char (match-beginning 2))
   (insert-and-inherit msg))
 
-(defsubst mew-syntax-change-parent-id (id)
+(defun mew-syntax-change-parent-id (id)
   (save-excursion
     (when (mew-sumsyn-match mew-regex-sumsyn-long)
       (delete-region (match-beginning 4) (match-end 4))
@@ -478,7 +459,7 @@ If this is a cache message, both the unique id and the size are also shown."
 ;; mew-encode-syntax
 ;;
 
-(defun mew-encode-syntax-single (file &optional ctl cte cd cid cdp 
+(defun mew-encode-syntax-single (file &optional ctl cte cd cid cdp
 				      privacy decrypters)
   ;; cid is just for beauty
   ;; if cdp is non-nil, set cdp from file.
@@ -514,13 +495,14 @@ If this is a cache message, both the unique id and the size are also shown."
    (list (mew-encode-syntax-text))))
 
 (defun mew-encode-syntax-initial-multi (dir n)
-  (let ((i 1) (ret))
+  (let ((i 1) file ret)
     (while (<= i n)
+      (setq file (mew-msg-new-filename (number-to-string i)))
       (setq ret (vconcat ret (list (mew-encode-syntax-single
-				    (int-to-string i) nil nil nil nil 'cdp))))
+				    file nil nil nil nil 'cdp))))
       (setq i (1+ i)))
     (vconcat (mew-encode-syntax-multi dir mew-type-mlm)
-	     (list (mew-encode-syntax-single mew-draft-coverpage 
+	     (list (mew-encode-syntax-single mew-draft-coverpage
 					     (list mew-ct-txt)))
 	     ret)))
 
@@ -554,7 +536,7 @@ If this is a cache message, both the unique id and the size are also shown."
 (defun mew-decode-syntax-text ()
   (vector 'single (point) (point-max) nil mew-type-txt nil nil nil nil))
 
-(defun mew-decode-syntax-multi-head () 
+(defun mew-decode-syntax-multi-head ()
   (vector 'multi nil nil nil mew-type-mlm nil nil nil nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -591,19 +573,10 @@ If this is a cache message, both the unique id and the size are also shown."
        (mew-xinfo-set-multi-form nil)
        (mew-xinfo-set-icon-spec nil)
        (mew-syntax-multipart syntax nil nil 'mew-draft-button)
-       (mapcar 'insert-and-inherit (nreverse (mew-xinfo-get-multi-form)))
-       (when mew-xemacs-p
-	 (goto-char (mew-attach-begin))
-	 (insert "X"))
+       (mapc 'insert-and-inherit (nreverse (mew-xinfo-get-multi-form)))
        (put-text-property (mew-attach-begin) (point-max) 'read-only t)
        (mew-front-nonsticky (mew-attach-begin) (1+ (mew-attach-begin)))
        (mew-rear-sticky (1- (point-max)) (point-max))
-       (when mew-xemacs-p
-	 (goto-char (1- (mew-attach-begin)))
-	 (delete-char 1))
-       (if (and mew-xemacs-p mew-icon-p)
-	   (mew-syntax-print-icon-spec (nreverse (mew-xinfo-get-icon-spec))
-				       mew-draft-toolbar))
        (mew-attach-goto-number 'here nums)))))
 
 ;;
@@ -626,13 +599,11 @@ If this is a cache message, both the unique id and the size are also shown."
 	(mew-elet
 	 (let ((pos (point)))
 	   (mew-decode-syntax-begin-set)
-	   (mapcar 'mew-decode-syntax-insert form)
+	   (mapc 'mew-decode-syntax-insert form)
 	   (if (= pos (point))
 	       ;; Nothing was printed.
 	       (mew-decode-syntax-remove)
 	     (mew-decode-syntax-end-set)))
-	 (if (and mew-xemacs-p mew-icon-p)
-	     (mew-syntax-print-icon-spec icon mew-summary-toolbar))
 	 (forward-line -1)
 	 (mew-summary-goto-message)
 	 (set-buffer-modified-p nil))))))
@@ -661,7 +632,6 @@ If this is a cache message, both the unique id and the size are also shown."
 
 (defun mew-syntax-multipart (syntax dec part func &optional body)
   (let* ((ct (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-	 (cd (or (mew-syntax-get-cd syntax) ""))
 	 (cnt mew-syntax-magic)
 	 (num 1)
 	 (len (length syntax))
@@ -674,14 +644,11 @@ If this is a cache message, both the unique id and the size are also shown."
 	 (concat (if part (concat part " "))
 		 (if body "body ")
 		 "multi"))
-      (mew-syntax-format syntax part dec)
-      (mew-syntax-set-icon-spec part
-				ct cd
-				(mew-ctdb-icon (mew-ctdb-by-ct ct)) func))
+      (mew-syntax-format syntax part dec))
     (while (< cnt len)
       (if part
-	  (setq strpart (concat part "." (int-to-string num)))
-	(setq strpart (int-to-string num)))
+	  (setq strpart (concat part "." (number-to-string num)))
+	(setq strpart (number-to-string num)))
       (setq subsyntax (aref syntax cnt))
       (cond
        ((mew-syntax-multipart-p subsyntax)
@@ -693,17 +660,15 @@ If this is a cache message, both the unique id and the size are also shown."
 			       (and body (= cnt mew-syntax-magic)))))
       (setq cnt (1+ cnt))
       (setq num (1+ num)))
-    (unless dec 
+    (unless dec
       (if part
-	  (setq part (concat part "." (int-to-string num)))
-	(setq part (int-to-string num)))
-      (mew-syntax-format mew-encode-syntax-dot part dec)
-      (mew-syntax-set-icon-spec part "Attach Here" cd mew-icon-blank func))))
+	  (setq part (concat part "." (number-to-string num)))
+	(setq part (number-to-string num)))
+      (mew-syntax-format mew-encode-syntax-dot part dec))))
 
 (defun mew-syntax-singlepart (syntax dec part func first)
   ;; part is valid only when called by mew-syntax-multipart.
-  (let ((ct (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-	(cd (or (mew-syntax-get-cd syntax) "")))
+  (let ((ct (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap)))
     ;; see also mew-mime-message/rfc822.
     (if (and dec
 	     ;; the first singlepart in multipart under message if t
@@ -714,9 +679,7 @@ If this is a cache message, both the unique id and the size are also shown."
 	     (mew-ct-textp ct))
 	() ;; skip displaying.
       ;; reach here only when called by mew-syntax-multipart.
-      (mew-syntax-format syntax part dec)
-      (mew-syntax-set-icon-spec part ct cd 
-				(mew-ctdb-icon (mew-ctdb-by-ct ct)) func))
+      (mew-syntax-format syntax part dec))
     (if dec (mew-decode-syntax-set-privacy
 	     syntax
 	     (if (eq first 'body)
@@ -724,15 +687,11 @@ If this is a cache message, both the unique id and the size are also shown."
 	       part)))))
 
 (defun mew-syntax-message (syntax dec part func)
-  (let ((ct (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-	(cd (or (mew-syntax-get-cd syntax) ""))
-	(subsyntax (mew-syntax-get-part syntax)))
+  (let ((subsyntax (mew-syntax-get-part syntax)))
     (mew-syntax-format syntax part dec)
     (if dec (mew-decode-syntax-set-privacy
 	     syntax
 	     (format "%s message" part)))
-    (mew-syntax-set-icon-spec
-     part ct cd (mew-ctdb-icon (mew-ctdb-by-ct ct)) func)
     (cond
      ((mew-syntax-multipart-p subsyntax)
       (mew-syntax-multipart subsyntax dec part func 'body))
@@ -760,7 +719,7 @@ If this is a cache message, both the unique id and the size are also shown."
 			  (mew-join "," decrypters-list)))
 	 (cd-or-dec cd)
 	 (privacy (mew-syntax-get-privacy syntax))
-	 (space " ") (LT (- (min mew-window-magic (window-width)) 2))
+	 (space " ") (LT (- (max mew-window-magic (window-width)) 2))
 	 (ln (length number))
 	 (lm 4) (lt 27)
 	 (ld (* (/ (- LT lm lt) 5) 3))
@@ -794,19 +753,21 @@ If this is a cache message, both the unique id and the size are also shown."
 	   ((string= cte mew-b64) (aset marks 0 ?B))
 	   ((string= cte mew-qp)  (aset marks 0 ?Q))
 	   ((string= cte mew-xg)  (aset marks 0 ?G))
-	   (t                     (aset marks 0 ?X))))
-      
+	   (t                     (aset marks 0 ??))))
+
       (if dec (setq privacy (reverse privacy)))
       (while (< i N)
 	(mew-set '(ctm ctp) (nth i privacy))
 	(let ((case-fold-search t))
 	  (cond
 	   ((string-match "pgp"   ctp) (aset marks (* i 2) ?P))
-	   ((string-match "pkcs"  ctp) (aset marks (* i 2) ?S)) ;; S/MIME
+	   ((string-match "pkcs"  ctm) (aset marks (* i 2) ?S))
 	   ((string-match "moss"  ctp) (aset marks (* i 2) ?M)))
 	  (cond
 	   ((mew-case-equal mew-ct-mle ctm) (aset marks (1+ (* i 2)) ?E))
-	   ((mew-case-equal mew-ct-mls ctm) (aset marks (1+ (* i 2)) ?S)))
+	   ((mew-case-equal mew-ct-mls ctm) (aset marks (1+ (* i 2)) ?S))
+	   ((mew-case-equal "enveloped-data" ctp) (aset marks (1+ (* i 2)) ?E))
+	   ((mew-case-equal "signed-data" ctp) (aset marks (1+ (* i 2)) ?S)))
 	  (setq i (1+ i)))))
 
     (if (and (not dec) decrypters) (setq cd-or-dec decrypters))
@@ -832,7 +793,7 @@ If this is a cache message, both the unique id and the size are also shown."
 	    "\n")
 	   (mew-xinfo-get-multi-form)))))
 
-(defsubst mew-decode-syntax-buffer ()
+(defun mew-decode-syntax-buffer ()
   (marker-buffer mew-marker-decode-syntax-begin))
 
 (defun mew-decode-syntax-delete ()
@@ -848,42 +809,15 @@ If this is a cache message, both the unique id and the size are also shown."
       (mew-highlight-cursor-line)
       (set-buffer-modified-p nil))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; icon spec
-;;
-
-(defun mew-syntax-set-icon-spec (part ct cd icon func)
-  (when (and mew-xemacs-p mew-icon-p)
-    (mew-xinfo-set-icon-spec
-     (cons
-      (vector
-       icon func t
-       (format
-	"%s <%s> (%s) %s" (mew-ainfo-get-icon) (or part "top") ct cd))
-      (mew-xinfo-get-icon-spec)))))
-
-(defun mew-syntax-print-icon-spec (spec bar)
-  (let ((toolbar))
-    (cond
-     ((eq mew-multipart-icon-position 'left)
-      (setq toolbar (append spec mew-icon-separate-spec bar)))
-     ((eq mew-multipart-icon-position 'right)
-      (setq toolbar (append bar mew-icon-separate-spec spec)))
-     (t (setq toolbar bar)))
-    (set-specifier default-toolbar (cons (current-buffer) toolbar))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; decode privacy
 ;;
 
 (defun mew-decode-syntax-set-privacy (syntax label)
-  (let ((privacy (mew-syntax-get-privacy syntax))
-	results)
-    (while privacy
-      (setq results (concat results (nth 2 (car privacy))))
-      (setq privacy (cdr privacy)))
+  (let (results)
+    (dolist (privacy (mew-syntax-get-privacy syntax))
+      (setq results (concat results (nth 2 privacy))))
     (when results
       (mew-xinfo-set-pri-result
        (concat (mew-xinfo-get-pri-result)
@@ -894,14 +828,15 @@ If this is a cache message, both the unique id and the size are also shown."
     (let ((beg (point)))
       (insert (mew-xinfo-get-pri-result))
       (mew-decode-header-property-region beg (point))
-      (save-restriction
-	(narrow-to-region beg (point))
-	(goto-char (point-min))
-	(while (re-search-forward "BAD.*sign" nil t)
-	  (put-text-property
-	   (match-beginning 0)
-	   (match-end 0)
-	   'face 'mew-face-header-xmew-bad))))))
+      (save-excursion
+	(save-restriction
+	  (narrow-to-region beg (point))
+	  (goto-char (point-min))
+	  (while (re-search-forward "BAD.*sign" nil t)
+	    (put-text-property
+	     (match-beginning 0)
+	     (match-end 0)
+	     'face 'mew-face-header-xmew-bad)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -935,51 +870,51 @@ If this is a cache message, both the unique id and the size are also shown."
 
 ;; location
 
-(defsubst mew-in-decode-syntax-p ()
+(defun mew-in-decode-syntax-p ()
   (and (marker-position mew-marker-decode-syntax-begin)
        (marker-position mew-marker-decode-syntax-end)
        (eq (current-buffer) (marker-buffer mew-marker-decode-syntax-begin))
        (>= (point) (marker-position mew-marker-decode-syntax-begin))
        (<  (point) (marker-position mew-marker-decode-syntax-end))))
 
-(defsubst mew-in-header-p ()
+(defun mew-in-header-p ()
   (let ((end (mew-header-end)))
     (and end (<= (point) end))))
 
-(defsubst mew-in-attach-p ()
+(defun mew-in-attach-p ()
   (let ((beg (mew-attach-begin)))
     (and beg (> (point) beg)))) ;; excluding the preceding \n
 
 ;; existence
 
-(defsubst mew-decode-syntax-p ()
+(defun mew-decode-syntax-p ()
   (and (marker-position mew-marker-decode-syntax-begin)
        (marker-position mew-marker-decode-syntax-end)))
 
-(defsubst mew-msghder-p ()
+(defun mew-msghdr-p ()
   (save-restriction
     (widen)
     (next-single-property-change (point-min) 'read-only)))
 
-(defsubst mew-attach-p ()
+(defun mew-attach-p ()
   (if (/= (point-max) 1)
       (get-text-property (1- (point-max)) 'mew-attach-end)))
 
-(defsubst mew-encode-syntax-have-one-part ()
+(defun mew-encode-syntax-have-one-part ()
   (= (length mew-encode-syntax) (1+ mew-syntax-magic)))
 
 ;; point
 
-(defsubst mew-decode-syntax-begin ()
+(defun mew-decode-syntax-begin ()
   (marker-position mew-marker-decode-syntax-begin))
 
-(defsubst mew-decode-syntax-end ()
+(defun mew-decode-syntax-end ()
   (marker-position mew-marker-decode-syntax-end))
 
-(defsubst mew-header-end ()
-  (mew-msghder-p))
+(defun mew-header-end ()
+  (mew-msghdr-p))
 
-(defsubst mew-attach-begin ()
+(defun mew-attach-begin ()
   (if (mew-attach-p)
       (let ((beg (previous-single-property-change
 		  (point-max) 'mew-attach-begin)))
@@ -987,13 +922,13 @@ If this is a cache message, both the unique id and the size are also shown."
 
 ;;
 
-(defsubst mew-decode-syntax-begin-set ()
+(defun mew-decode-syntax-begin-set ()
   (set-marker mew-marker-decode-syntax-begin (point)))
 
-(defsubst mew-decode-syntax-end-set ()
+(defun mew-decode-syntax-end-set ()
   (set-marker mew-marker-decode-syntax-end (point)))
 
-(defsubst mew-decode-syntax-remove ()
+(defun mew-decode-syntax-remove ()
   (set-marker mew-marker-decode-syntax-begin nil)
   (set-marker mew-marker-decode-syntax-end nil))
 
@@ -1001,17 +936,18 @@ If this is a cache message, both the unique id and the size are also shown."
   (let (pos beg end)
     (save-excursion
       (set-buffer (mew-decode-syntax-buffer))
-      (setq pos (point))
-      (setq end (mew-decode-syntax-end))
-      (goto-char end)
-      (forward-line -1)
-      (beginning-of-line)
-      (setq beg (point))
-      (and (<= beg pos) (< pos end)))))
+      (save-excursion
+	(setq pos (point))
+	(setq end (mew-decode-syntax-end))
+	(goto-char end)
+	(forward-line -1)
+	(beginning-of-line)
+	(setq beg (point))
+	(and (<= beg pos) (< pos end))))))
 
 ;;
 
-(defsubst mew-header-set (sep)
+(defun mew-header-set (sep)
   (mew-elet
    (let ((pos (point)))
      (if sep
@@ -1026,7 +962,7 @@ If this is a cache message, both the unique id and the size are also shown."
        (mew-rear-nonsticky (1- (point)) (point)))
      pos)))
 
-(defsubst mew-header-clear (&optional keep-read-only)
+(defun mew-header-clear (&optional keep-read-only)
   ;; the cursor moves to the end of the header (with some exceptions)
   (mew-elet
    (mew-header-goto-end) ;; do not use mew-header-end
@@ -1041,7 +977,7 @@ If this is a cache message, both the unique id and the size are also shown."
 
 ;;
 
-(defsubst mew-attach-set ()
+(defun mew-attach-set ()
   (mew-elet
    (let (beg)
      (goto-char (point-max))
@@ -1057,7 +993,7 @@ If this is a cache message, both the unique id and the size are also shown."
      (beginning-of-line)
      (mew-draft-attach-keymap))))
 
-(defsubst mew-attach-clear ()
+(defun mew-attach-clear ()
   (when (mew-attach-p)
     (save-excursion
       (mew-elet
@@ -1065,12 +1001,12 @@ If this is a cache message, both the unique id and the size are also shown."
       (mew-overlay-delete (mew-tinfo-get-attach-keymap))
       (mew-tinfo-set-attach-keymap nil))))
 
-(defsubst mew-header-prepared ()
+(defun mew-header-prepared ()
   (mew-header-set (concat mew-header-separator "\n"))
   (mew-highlight-header)
   (unless (mew-header-p) (mew-draft-header-keymap)))
 
-(defsubst mew-draft-header-keymap ()
+(defun mew-draft-header-keymap ()
   (save-excursion
     (if (overlayp (mew-tinfo-get-header-keymap))
 	(move-overlay
@@ -1078,28 +1014,28 @@ If this is a cache message, both the unique id and the size are also shown."
       (mew-tinfo-set-header-keymap
        (mew-overlay-make (point-min) (1+ (mew-header-end))))
       (overlay-put
-       (mew-tinfo-get-header-keymap) mew-local-map mew-draft-header-map))))
+       (mew-tinfo-get-header-keymap) 'local-map mew-draft-header-map))))
 
-(defsubst mew-draft-attach-keymap ()
+(defun mew-draft-attach-keymap ()
   (if (overlayp (mew-tinfo-get-attach-keymap))
       (move-overlay
        (mew-tinfo-get-attach-keymap) (1+ (mew-attach-begin)) (point-max))
     (mew-tinfo-set-attach-keymap
      (mew-overlay-make (1+ (mew-attach-begin)) (point-max)))
     (overlay-put
-     (mew-tinfo-get-attach-keymap) mew-local-map mew-draft-attach-map)))
+     (mew-tinfo-get-attach-keymap) 'local-map mew-draft-attach-map)))
 
 (provide 'mew-syntax)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1108,7 +1044,7 @@ If this is a cache message, both the unique id and the size are also shown."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
