@@ -1868,11 +1868,15 @@ If you cite a message, the citation style of format=flowed is used. "
            (list (const message) (list integer integer))
            (list (const draft) (list integer integer))))
 
-(defcustom mew-mode-line-id '("Mew: %12b")
+(defcustom mew-mode-line-id "Mew: %12b"
   "*A default value of mode-line-buffer-identification for each Mew mode."
   :group 'mew-summary
   :type 'sexp)
 
+;; Indirect call of propertized-buffer-identification due to XEmacs
+(defun mew-mode-line-id ()
+  (propertized-buffer-identification mew-mode-line-id))
+
 (defcustom mew-multipart-icon-position 'right
   "*Position where multipart icons are displayed.
 If 'left, displayed at the left size of the default toolbar.
@@ -2683,7 +2687,7 @@ guys can wiretap the temporary file."
 (defcustom mew-auto-get t
   "If *non-nil*, Mew gets messages from your mailbox automatically.
 '\\[universal-argument] \\[mew]' equals to '\\[mew]' with 'mew-auto-get'
-revered."
+reversed."
   :group 'mew-summary
   :type 'boolean)
 
@@ -3265,7 +3269,7 @@ Level 2: syntax error."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Preventing waring messages when byte-compiling
+;;; Preventing warning messages when byte-compiling
 ;;;
 
 (defvar mew-regex-message-files  nil)
