@@ -6,12 +6,12 @@
 ;;; Code:
 
 (eval-when-compile
-  (mew-no-waring-defvar w32-start-process-show-window)
-  (mew-no-waring-defun mw32script-init)
-  (mew-no-waring-defun mw32script-make-pathext-regexp)
-  (mew-no-waring-defun mw32script-openp)
-  (mew-no-waring-defun mw32script-resolve-script)
-  (mew-no-waring-defun dos-to-unix-filename))
+  (mew-no-warning-defvar w32-start-process-show-window)
+  (mew-no-warning-defun mw32script-init)
+  (mew-no-warning-defun mw32script-make-pathext-regexp)
+  (mew-no-warning-defun mw32script-openp)
+  (mew-no-warning-defun mw32script-argument-editing-function)
+  (mew-no-warning-defun dos-to-unix-filename))
 
 ;;; for NTEmacs User
 ;;
@@ -59,34 +59,42 @@
   (require 'mw32script)
   (mw32script-init))
  ((condition-case nil
-      (require 'mw32script)
+      (progn
+	(require 'mw32script)
+	(load "mw32misc"))
     (file-error nil)) ;; NTEmacs
   (mw32script-make-pathext-regexp)
-  (defalias 'call-process-original 'call-process)
-  (defun call-process (PROGRAM INFILE BUFFER DISPLAY &rest PROGRAM-ARGS)
-    (let (prog sargs)
-      (setq prog (mw32script-openp PROGRAM))
-      (unless prog
-	(setq prog (mew-which-exec PROGRAM))
-	(setq sargs (mw32script-resolve-script prog)))
-      (if sargs
-	  (apply 'call-process-original
-		 (car sargs) INFILE BUFFER DISPLAY
-		 prog PROGRAM-ARGS)
-	(apply 'call-process-original
-	       PROGRAM INFILE BUFFER DISPLAY PROGRAM-ARGS))))
-  (defalias 'start-process-original 'start-process)
-  (defun start-process (NAME BUFFER PROGRAM &rest PROGRAM-ARGS)
-    (let (prog sargs)
-      (setq prog (mw32script-openp PROGRAM))
-      (unless prog
-	(setq prog (mew-which-exec PROGRAM))
-	(setq sargs (mw32script-resolve-script prog)))
-      (if sargs
-	  (apply 'start-process-original
-		 NAME BUFFER (car sargs) prog PROGRAM-ARGS)
-	(apply 'start-process-original
-	       NAME BUFFER PROGRAM PROGRAM-ARGS))))))
+  (defun mew-w32-argument-editing-function (program args)
+    (let ((default-process-argument-editing-function 'identity)
+	  (process-argument-editing-alist nil)
+	  prog sargs)
+      (setq prog (mw32script-openp program))
+      (if (and (null prog)
+	       (setq prog (mew-which-exec program))
+	       (setq sargs (mw32script-argument-editing-function (list prog))))
+	  (cons (car sargs) (cons prog args))
+	(cons program args))))
+  (defadvice call-process 
+    (before mew-w32-call-process 
+	    (program &optional infile buffer display &rest args)
+	    activate)
+    (let ((sargs (mew-w32-argument-editing-function program args)))
+      (setq program (car sargs)
+	    args (cdr sargs))))
+  (defadvice call-process-region
+    (before mew-w32-call-process-region 
+	    (start end program &optional infile buffer display &rest args)
+	    activate)
+    (let ((sargs (mew-w32-argument-editing-function program args)))
+      (setq program (car sargs)
+	    args (cdr sargs))))
+  (defadvice start-process 
+    (before mew-w32-start-process
+	    (name buffer program &rest program-args)
+	    activate)
+    (let ((sargs (mew-w32-argument-editing-function program program-args)))
+      (setq program (car sargs)
+	    program-args (cdr sargs))))))
 
 ;; printing
 (defun mew-w32-print-buffer ()
@@ -202,7 +210,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2009 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
